/***************************************************************
 * Name:      bil31core.h
 * Purpose:   Core Code for Bilancio Application Class
 * Author:    Igor Calì (igor.cali0@gmail.com)
 * Created:   2016-12-09
 * Copyright: Igor Calì (http://igisw-bilancio.sourceforge.net/)
 * License:		GNU
 **************************************************************/

#ifndef BIL31CORE_HEADER_INCLUDED
#define BIL31CORE_HEADER_INCLUDED

#ifndef WX_PRECOMP
	#include <wx/wx.h>
#else
	#include <wx/wxprec.h>
#endif

#include <wx/string.h>
#ifndef __BILCONVERT_BIN__
	#include <wx/gauge.h>
#endif
#include <wx/fileconf.h> // For wxFileConfig
#include <wx/wxsqlite3.h>

#if(! wxCHECK_VERSION(3,0,0))
	#include <wx/datetime.h>
#endif

#include "types.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#ifndef Bil31CoreH
#define Bil31CoreH

// Product Capabilities
const int MAX_FUNDS     	=   100;
const int MAX_CREDITS   	=   100;
const int MAX_DEBTS     	=   100;
const int MAX_LENT      	=  1000;
const int MAX_BORROWED  	=  1000;
const int MAX_SHOPS     	=  1000;
const int MAX_LINES     	= 10000;
const int MAX_CATEGORIES	=   100;
//______________________________

typedef enum TTypeVal{tvFou=1,tvCre,tvDeb}TypeVal;
typedef struct TVal{
	int Id;
	wxString Name;
	double Value;

	/* TODO -cImprovement : To be implemented
	TDate CreationDate;
	TDate DeletionDate;
	*/
}Val;

typedef enum TObjType{toPre=1,toInP}ObjType;
typedef struct TObj{
	int Id;
	wxString Name, Object;
	wxDateTime Alarm;

	/* TODO -cImprovement : To be implemented
	wxDateTime CreationDate;
	wxDateTime DeletionDate;
	*/
}Obj;

typedef struct TShopItem{
	int Id;
	wxString Name;
	wxDateTime Alarm;}ShopItem;

typedef enum TOpType{
  toNULL=0,
  toGain=1,toExpe,
  toSetCre,toRemCre,toConCre,
  toSetDeb,toRemDeb,toConDeb,
  toGetObj,toGivObj,
  toLenObj,toBakObj,
  toBorObj,toRetObj}OpType;

typedef struct TLine{
	int Id;
	bool IsDate;
	wxDateTime Date,Time;
	TOpType Type;
	wxString Value,
	// FIXME (igor#1#):	Known limitation: Value is defined as WxString also in case it contains numeric data. ...
	//									So it is saved as a locale-dependent string.
	Reason;
	long CategoryIndex;
	#ifdef __BILCONVERT_BIN__
		wxString Remark0301;	// "Bil3.0"
	#endif // __BILCONVERT_BIN__
	}Line;

// Database information
enum dbMetaDataEntry{
	dbMeta_application_info = 1,
	dbMeta_default_fund,
	dbMeta_mobile_export
};

typedef struct TCategory{
	int Id;
	wxString Name;
	bool test;	// used during category update
	}Category;

class TData
{
friend class BilancioMainFrame;

private:
	// Non-GUI components

	double Tot_Credits,Tot_Debts;

	// Routines
	//bool SortValues(TTypeVal T);
	//void CheckMemory(void); // NOTE (igor#1#): Do not remove, may be restored in future
	void ParseDatabase(void);
	//wxString BackupName(void);
	bool Prepare_MasterDB(wxString dbPath, wxString trailname);
	void Archive_inMaster(wxString master, wxString trailname, bool archive_categories);
protected:
	// Non-GUI components
	double Tot_Funds;

	// Routines
	#ifndef __BILCONVERT_BIN__
		void UpdateProgress(int i);
	#endif // __BILCONVERT_BIN__

	#if(wxCHECK_VERSION(3, 0, 0))
		wxString bilFromCDouble(double value);
	#endif
	//void Sort(void);
public:
	// Non-GUI components
	//bool NotEnoughMemory;
	wxSQLite3Database *database;

	bool Parsing; // Set to true during file parsing
	wxDateTime Day;
	wxDateTime Hour;
	int NFun, NCre, NDeb, NLen, NBor, NSho, NLin, NCat;
	struct ACC{
		bool	Modified,
					ReadOnly;
		int Year;
		wxDateTime DateStamp;
		long Month;
		wxString	FileName,
							FileView,
							DefFund;}FileData;
	TVal *Funds,*Credits,*Debts;
	TObj *Lent,*Borrowed;
	TShopItem *ShopItems;
	TLine *Lines;
	TCategory *Categories;
	wxArrayString *MattersBuffer;
	#ifdef __BILCONVERT_BIN__
		wxArrayString *CategoryDB;
		TData(void);

	#else
		wxGauge *Progress;
		TData(wxGauge *ProgressBar);
	#endif // __BILCONVERT_BIN__

	// Routines
	~TData(void);
	void Initialize(bool Creating);
	bool AddDate(int id, wxDateTime D, double T);
	bool AddValue(TTypeVal T, int id, wxString N, double V);
	bool DelValue(TTypeVal T,int I);
	bool AddObject(TObjType T, int id, wxString N, wxString O, wxDateTime D);
	bool DelObject(TObjType T,int I);
	bool AddShopItem(int id, wxString N, wxDateTime A);
	bool DelShopItem(int I);
	bool AddOper(int id, wxDateTime D, wxDateTime O, TOpType T, wxString V, wxString M, long N);
	#ifndef __BILCONVERT_BIN__
		//void ModPass(void);	// TODO (igor#1#): To be restored when SQL encryption is available
		void XMLExport(wxString F1, wxString F2);
		void XMLExport_archive(wxString F1, wxString F2, wxDateTime date);
		void UpdateCategories(wxListBox *lbox);
	#endif // __BILCONVERT_BIN__
	double GetTot(TTypeVal T);
	//bool SetTot(TTypeVal T,double Tot);
	bool IsDate(int R);
	bool UpdateMatters(TOpType T);
	bool OpenDatabase(wxString File);
	void SetDefaultFundValue(double value);
	void SetDefaultFund(wxString def);
	bool ChangeFundValue(TTypeVal type, int id, /*wxString N,*/ double V);
	void AddCategory(int id, wxString name);
	void ChangeTransactionCategory(int id, int new_index);
	//void RestoreBackup(void);
	wxString GenerateTrailName(wxString dbPath);
	void Import_inMaster(wxString master, wxString trailname);
};

#ifndef __BILCONVERT_BIN__
	WXIMPORT wxString FormDigits(double Val);
	WXIMPORT wxDateTime Bil_StrToDate(wxString S);
	WXIMPORT wxString IntToMonth(int M);
	WXIMPORT void Error(int Err,wxString Opt);
	WXIMPORT bool AutoConvert(void);
	WXIMPORT wxString GetLogString(int I);
	WXIMPORT wxString SubstSpecialChars(wxString S);
	WXIMPORT int CheckFileFormat(wxString File);
	WXIMPORT wxString GetBilDocPath(void);
	WXIMPORT wxString GetDataDir(void);
	WXIMPORT wxString GetOSDocDir(void);
	WXIMPORT bool CheckAndPromptForConversion(wxString File);
	#ifdef __WXMSW__
		WXIMPORT wxLanguage FindLang(void);
		WXIMPORT wxString GetInstallationPath(void);
	#endif //__WXMSW__

	/*
	// TODO (igor#1#): To be restored when SQL encryption is available
	WXIMPORT wxString Get_Key(void);
	WXIMPORT bool Set_Key(wxString Key);
	*/
	WXIMPORT wxString Get_MasterDB(void);

	#if (! wxCHECK_VERSION(3, 0, 0) )
		WXIMPORT bool SubstLocaleDecimalSeparator(wxString &S, char ToSub);
	#endif

	WXIMPORT wxString DoubleQuote(wxString S);

#else
	extern wxString Crip(wxString S);
	extern wxString FormDigits(double Val);
	extern wxDateTime Bil_StrToDate(wxString S);
	extern void Error(int Err,wxString Opt);
	extern int CheckFileFormat(wxString File);

	#if (! wxCHECK_VERSION(3, 0, 0) )
		extern bool SubstLocaleDecimalSeparator(wxString &S, char ToSub);
	#endif

	extern wxString DoubleQuote(wxString S);
#endif // __BILCONVERT_BIN__

#endif // Bil31CoreH

#endif  // BIL31CORE_HEADER_INCLUDED
