/***************************************************************
 * Name:      bilancio.cpp
 * Purpose:   Code for Bilancio Application Class
 * Author:    Igor Cal (igor.cali0@gmail.com)
 * Created:   2017-01-08
 * Copyright: Igor Cal (igor.cali0@gmail.com)
 * License:		GNU
 **************************************************************/

#ifndef BILANCIO_CPP_INCLUDED
#define BILANCIO_CPP_INCLUDED

#ifndef WX_PRECOMP
	#include <wx/wx.h>
#else
	#include <wx/wxprec.h>
#endif

#ifndef __WXMSW__
  #include <signal.h>
#endif // __WXMSW__

#include <wx/app.h>
//#include <wx/cmdline.h>
#include <wx/fileconf.h> // For wxFileConfig
#include <wx/filename.h>    // For wxFileName
#include <wx/intl.h> // For i18n
#include <wx/snglinst.h>	// for wxSingleInstanceChecker
#include <wx/stdpaths.h> // For wxStandardPathsBase

#ifdef __WXGTK__
	#include "../platformsetup.h"
#endif // __WXGTK__
#include "../types.h"
#include "bilancio.h"
//#include "constants.h"
//#include "igibil.h"
#include "ui/main_wx.h"
//#include "igiberr.h"
//#include "../bil30opt.h"
//#include "../productversion.h"

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

// Code start

static const wxLanguage langIds[]={
	wxLANGUAGE_DEFAULT,
	wxLANGUAGE_ENGLISH,
	wxLANGUAGE_ITALIAN};

const wxString langNames[]={
	_("System language"),
	L"English",
  L"Italiano"};

// the arrays must be in sync
wxCOMPILE_TIME_ASSERT( WXSIZEOF(langNames)==WXSIZEOF(langIds),LangArraysMismatch);

#ifndef __WXMSW__
  bool Killed=false;	// Set to true when SIGTERM is received
  wxTimer *CheckTimer;
#endif // __WXMSW__
wxLanguage Lan;

wxLocale bil_locale;
#ifndef __WXMSW__
  wxFileConfig *INI;
#endif // __WXMSW__
TData *Data;
BilancioMainFrame *frame;
#ifndef __WXMSW__
  extern char __BUILD_NUMBER;
  extern char __BUILD_DATE;
#endif // __WXMSW__
FILE * m_pLogFile;

IMPLEMENT_APP(BilApp)

#ifndef __WXMSW__
  void term(int signum){	// Captures the SIGTERM signal
                          // http://airtower.wordpress.com/2010/06/16/catch-sigterm-exit-gracefully/
      Killed = true;}
#endif // __WXMSW__

bool BilApp::OnInit(){

	#ifdef __WXMSW__
    if (m_pLogFile == NULL)
    {
      m_pLogFile = fopen(GetInstallationPath() + L"\\bilancio.log", "w" );
      delete wxLog::SetActiveTarget(new wxLogStderr(m_pLogFile));
    }
    wxLogMessage("Program starts...");
	#else
		wxString home_dir = wxGetHomeDir();

		wxString homecfg = GetUserConfigDir();

		// Start logging
		if (m_pLogFile == NULL)
		{
			#ifdef __WXGTK__
				#if( wxCHECK_VERSION(3,0,0) )
					#ifndef __OPENSUSE__
						m_pLogFile = fopen( homecfg + "/bilancio.log", "w" );
					#else
						wxString f = homecfg + L"/bilancio.log";
						wxString am = L"w";
						m_pLogFile = fopen( f.fn_str(), am.fn_str() );
					#endif // __OPENSUSE__
				#else
					wxString f = homecfg + L"/bilancio.log";
					wxString am = L"w";
					m_pLogFile = fopen( f.fn_str(), am.fn_str() );
				#endif
			#endif // __WXGTK__
			delete wxLog::SetActiveTarget(new wxLogStderr(m_pLogFile));
		}
		wxLogMessage(L"Program starts...");


		homecfg+=L"/bilancio.ini";
		if(!::wxFileExists(homecfg)){
			wxFile *f=new wxFile(homecfg,wxFile::write);
			//f->Create(homecfg,false,wxS_DEFAULT);
			f->Close();}
		INI=new wxFileConfig(wxEmptyString,wxEmptyString,homecfg,wxEmptyString,wxCONFIG_USE_LOCAL_FILE,wxConvAuto());
	#endif // __WXMSW__

	#ifdef _BIL_DEBUG
		#ifdef __WXGTK__
      wxLocale::AddCatalogLookupPathPrefix(L".");
    #endif // __WXGTK__
	#endif

	#ifdef __WXMSW__
    wxLocale::AddCatalogLookupPathPrefix(GetShareDir());
	#else
    wxLocale::AddCatalogLookupPathPrefix(GetShareDir()+L"locale"/*/it/LC_MESSAGES"*/);
    		// NOTE (igor#1#): 2014/08/16 - Monitor the path at line above; it worked well previously with full path (now commented).
    		//                 Is it related to wxWidgets 3.0.1 installed in Ubuntu from tarball since debian packages not available?
	#endif // __WXMSW__
	bil_locale.Init(wxLANGUAGE_DEFAULT,wxLOCALE_LOAD_DEFAULT);
	bil_locale.AddCatalog(L"bilancio");
	Lan=FindLang();

  // Check if already running
  wxString user=wxGetUserId();
  const wxString name = wxString::Format(L"bilancio-%s",user.c_str());
	m_checker_bilancio = new wxSingleInstanceChecker(name);
	if (m_checker_bilancio->IsAnotherRunning()){
		wxMessageBox(_("Bilancio is already running!"),_("Bilancio"), wxICON_EXCLAMATION);
		return false;}

	#ifdef __WXMSW__
    bool frame_created = false;
  #endif // __WXMSW__
	const wxString name2 = wxString::Format(L"biltray-%s",user.c_str());
	wxSingleInstanceChecker *m_checker_biltray = new wxSingleInstanceChecker(name2);
	if (m_checker_biltray->IsAnotherRunning()){
		wxString Proc;
		#ifdef __WXMSW__
      long processid = 0;
      wxStandardPathsBase& Paths = ::wxStandardPaths::Get();
      wxString hm = Paths.GetTempDir() + L"\\bil";
      wxString tempfile = ::wxFileName::CreateTempFileName(hm);
      Proc = L"tasklist |find \"biltray.exe\" >" + tempfile;

      // Following lines are necessary to avoid Bilancio exits because wxShell is recognized as top window
      frame = new BilancioMainFrame(wxTheApp->GetTopWindow());
      frame_created = true;

      ::wxShell(Proc);
      if(::wxFileExists(tempfile)){
        int L, b;
        char *Buffer;
        wxString S = wxEmptyString;
        wxFile *file = new wxFile(tempfile, wxFile::read);
        L = file->SeekEnd(0);
        file->Seek(0);
        Buffer = new char[L + 1];
        file->Read(Buffer, L);

        b = 0;
        while ((Buffer[b] < char(48)) | (Buffer[b] > char(57))){
          b++;}
        do{
          S += wxChar(Buffer[b]);
          b++;}
        while ((Buffer[b] >= char(48)) & (Buffer[b] <= char(57)));

        S.ToLong(&processid);
        file->Close();
        ::wxRemoveFile(tempfile);
        delete m_checker_biltray;}
      if(processid > 0)wxKill(processid, wxSIGKILL);

		#else
			Proc = L"killall -u " + user + L" biltray";
      wxExecute(Proc,wxEXEC_ASYNC);
      bool biltray_killed = false;
      int s=0;
      while(s <= 60){
        if(m_checker_biltray->IsAnotherRunning()){
          wxSleep(5);
          delete m_checker_biltray;
          if(::wxFileExists(home_dir + L"/" + name2)) wxRemoveFile (home_dir + L"/" + name2);
          m_checker_biltray = new wxSingleInstanceChecker(name2);
          s += 5;}
        else{
          delete m_checker_biltray;
          biltray_killed = true;
          break;}}
      if(! biltray_killed) return false;
		#endif
  }
  else delete m_checker_biltray;
	wxLogMessage(L"Existing instances cleared");

 	// Document memory-space creation
	Data=new TData(NULL); // Progress bar is set after frame creation
	wxLogMessage(L"Data created");

  // Main Window creation
  #ifdef __WXMSW__
    if(! frame_created)
  #endif // __WXMSW__
	frame = new BilancioMainFrame(wxTheApp->GetTopWindow());
	wxLogMessage(L"Main window created");

  Data->Progress = frame->Progress;

	if(! frame->AutoOpen()){
		delete Data;
		delete m_checker_bilancio;
		wxLogMessage(L"Error opening main window!");
		Exit();
		return false;
	}
	wxLogMessage(L"Auto document opening done");

	Data->Hour = ::wxDateTime::Now();

	frame->UpdateCalendar(wxInvalidDateTime);

	frame->Centre(wxBOTH);
	frame->Show();
	#ifdef _BIL_CHART_MATLIBPLOT
		frame->ShowF();
	#endif // _BIL_CHART_MATLIBPLOT
	SetTopWindow(frame);

	wxLogMessage(L"Main window set to top");
	if(SplashActive())Logo(_("Bilancio"));

	#ifdef _BIL_INSTALLEDUPDATE
		if(GetCheckUpdates())fnCheckUpdate();	// Version update check
	#endif // _BIL_INSTALLEDUPDATE

	#ifndef __WXMSW__
    // Prepare timer to check SIGTERM
    Connect( wxEVT_TIMER, wxTimerEventHandler( BilApp::CheckTimerFire ) );
    CheckTimer=new wxTimer();
    CheckTimer->SetOwner(this);
    CheckTimer->Start(500,false);
  #endif // __WXMSW__

 	wxLogMessage(L"Initialization over... starting...");
	return true;}

#ifndef __WXMSW__
  void BilApp::CheckTimerFire( wxTimerEvent& event ){
    #ifdef __WXGTK__
      // Following code prepare the action to caputure SIGTERM signals
      struct sigaction action;
      memset(&action, 0, sizeof(struct sigaction));
      action.sa_handler = term;
      sigaction(SIGTERM, &action, NULL);
    /*
    #elif __WXMSW__
      typedef void (*SignalHandlerPointer)(int);
      SignalHandlerPointer previousHandler;
      previousHandler = signal(SIGABRT, term);
    */
    #endif

    //int loop = 0;
    if(!Killed){
        //sleep(3);
        //printf("Finished loop run %d.\n", loop++);
        return;}

		wxLogMessage(L"Kill request received.");

    //printf("done.\n");
    wxCloseEvent evt(wxEVT_CLOSE_WINDOW,0);
    frame->FormClose(evt);
    return;}
#endif // __WXMSW__

int BilApp::OnExit(){
	#ifndef __WXMSW__
    CheckTimer->Stop();
    delete CheckTimer;
  #endif // __WXMSW__
	delete Data;
	delete m_checker_bilancio;

	if(TrayActive()){
		wxString Proc = wxEmptyString;
    wxStandardPaths std
    	#if(wxCHECK_VERSION(3,0,0))
    		= wxStandardPaths::Get()
			#endif
		;
    wxString exePath = std.GetExecutablePath();
    wxString path;
    wxFileName::SplitPath (exePath, &path, NULL, NULL, wxPATH_NATIVE);

    #ifdef __WXMSW__
      Proc = "\"";
    #endif // __WXMSW__

    Proc = Proc + path;

    #ifdef __WXMSW__
      Proc = Proc + L"\\";
    #else
      Proc = Proc + L"/";
    #endif // __WXMSW__

    Proc = Proc + L"biltray";

    #ifdef __WXMSW__
      Proc = Proc + L".exe\"";
    #endif // __WXMSW__

		wxExecute(Proc,wxEXEC_ASYNC);}

	#ifndef _BIL_USEREGISTRY
    delete INI;
  #endif // _BIL_USEREGISTRY

	wxLogMessage(L"Program exiting...");
	delete wxLog::SetActiveTarget(NULL);
	if (m_pLogFile != NULL)
	{
		fclose(m_pLogFile);
	}

	return 0;}

#endif	// BILANCIO_CPP_INCLUDED
