/***************************************************************
 * Name:      main_wx.cpp
 * Purpose:   Code for Bilancio Application Class
 * Author:    Igor Calì (igor.cali0@gmail.com)
 * Created:   2017-01-11
 * Copyright: Igor Calì (igor.cali0@gmail.com)
 * License:		GNU
 **************************************************************/

#ifndef MAINFRAME_CPP_INCLUDED
#define MAINFRAME_CPP_INCLUDED

#ifndef WX_PRECOMP
	#include <wx/wx.h>
#else
	#include <wx/wxprec.h>
#endif

#include <wx/imaglist.h> // For wxImageList
//#include <wx/tokenzr.h>	// For wxStringTokenize
#include <wx/clipbrd.h> // For wxTheClipboard
#include <wx/filename.h>
#include <wx/gdicmn.h>
#include <wx/stdpaths.h>	// For wxStandardPaths::Get()

#include "../../platformsetup.h"

#ifdef _BIL_CHART_WXFREECHART
	#include <wx/pie/pieplot.h>	// For wxFreeChart components
	#include <wx/category/categorysimpledataset.h>
	#include <wx/chartpanel.h>
	#include <wx/xy/xyplot.h>
#endif // _BIL_CHART_WXFREECHART

#if(!wxCHECK_VERSION(3,0,0))
	#include "../../3rdparty/mytimectrl.h"
#endif

#if (_BIL_ICONSIZE_ == 24)
	#ifdef _BIL_CHART_MATLIBPLOT
		#include "wxmainframe_24_mpl.h"
	#else
		#include "wxmainframe_24.h"
	#endif // _BIL_CHART_MATLIBPLOT
#else
	#include "wxmainframe_16.h"
#endif // _BIL_ICONSIZE_

#include "../../bil31core.h"

//#include "../../bil30opt.h"
#include "main_wx.h"
#include "funds.h"
#include "gainexpense.h"
#include "editcategories.h"
#include "setcreddeb.h"
#include "remcreddeb.h"
#include "objct.h"
//#include "contribform.h"
#include "getbackobj.h"
#include "../../constants.h"
#ifdef _BIL_CHART_WXFREECHART
	#include "../timedataset.h"
#endif // _BIL_CHART_WXFREECHART
//#include "../../productversion.h"

#ifndef __WXMSW__
  #include "../../../rsrc/icons/Icon_Application.xpm"
#endif

#ifdef __WXMSW__
  #include "../../../rsrc/toolbar/16/creditnew.xpm"
  #include "../../../rsrc/toolbar/16/debtnew.xpm"
  #include "../../../rsrc/toolbar/16/newshop.xpm"
  #include "../../../rsrc/toolbar/16/objectgiven.xpm"
  #include "../../../rsrc/toolbar/16/objectreceived.xpm"
  #include "../../../rsrc/toolbar/16/newfund.xpm"
#else
  #if(_BIL_ICONSIZE_ == 24)
    #include "../../../rsrc/toolbar/20/creditnew20.xpm"
    #include "../../../rsrc/toolbar/20/debtnew20.xpm"
    #include "../../../rsrc/toolbar/20/apple_red20.xpm"
    #include "../../../rsrc/toolbar/20/objectgiven20.xpm"
    #include "../../../rsrc/toolbar/20/objectreceived20.xpm"
    #include "../../../rsrc/toolbar/20/newfund20.xpm"
  #else
    #include "../../../rsrc/toolbar/16/creditnew.xpm"
    #include "../../../rsrc/toolbar/16/debtnew.xpm"
    #include "../../../rsrc/toolbar/16/newshop.xpm"
    #include "../../../rsrc/toolbar/16/objectgiven.xpm"
    #include "../../../rsrc/toolbar/16/objectreceived.xpm"
    #include "../../../rsrc/toolbar/16/newfund.xpm"
  #endif // _BIL_ICONSIZE
#endif // __WXMSW__

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

extern TData *Data;
extern wxLanguage Lan;

#ifdef __WXGTK__
	extern wxString Console;
#endif // __WXGTK__

BilancioMainFrame::BilancioMainFrame(wxWindow* parent):wxMainFrame(parent, wxID_ANY, _("Budget management"), wxDefaultPosition, wxSize(790, 630), wxDEFAULT_FRAME_STYLE | wxTAB_TRAVERSAL ){

	Master_Attached = false;

	Initialised=false;
	menu_opendoc=true;
	Report->SetSelectionMode(wxGrid::wxGridSelectRows);

	backup_report_colour = Report->GetDefaultCellTextColour();

	// Menu
	#if _BIL_INSTALLEDCONV == 0
		MainMenu_Tools_->Remove(bilOldConv);
	#endif // _BIL_INSTALLEDCONV

	MainMenu_File_->Remove(bilModifyPassword);
	MainMenu_File_->Remove(MainMenu_File_->FindItemByPosition(MainMenu_File_->GetMenuItemCount()-2)); // password_separator removal

	// GUI controls
	// TreeCtrl glyphs
  #ifdef __WXMSW__
    wxImageList *imlist=new wxImageList(16,16,true,0);
  #else
    wxImageList *imlist=new wxImageList();
  #endif // __WXMSW__
	imlist->RemoveAll();

  #ifdef __WXMSW__
		imlist->Add(wxBitmap(newfund_xpm),wxNullBitmap);
		imlist->Add(wxBitmap(creditnew_xpm),wxNullBitmap);
		imlist->Add(wxBitmap(debtnew_xpm),wxNullBitmap);
		imlist->Add(wxBitmap(newshop_xpm),wxNullBitmap);
		imlist->Add(wxBitmap(objectgiven_xpm),wxNullBitmap);
		imlist->Add(wxBitmap(objectreceived_xpm),wxNullBitmap);

  #else
    #if(_BIL_ICONSIZE_ == 24)
      imlist->Add(wxBITMAP(newfund20),wxNullBitmap);
      imlist->Add(wxBITMAP(creditnew20),wxNullBitmap);
      imlist->Add(wxBITMAP(debtnew20),wxNullBitmap);
      imlist->Add(wxBITMAP(apple_red20),wxNullBitmap);
      imlist->Add(wxBITMAP(objectgiven20),wxNullBitmap);
      imlist->Add(wxBITMAP(objectreceived20),wxNullBitmap);
    #else
      imlist->Add(wxBITMAP(newfund),wxNullBitmap);
      imlist->Add(wxBITMAP(creditnew),wxNullBitmap);
      imlist->Add(wxBITMAP(debtnew),wxNullBitmap);
      imlist->Add(wxBITMAP(newshop),wxNullBitmap);
      imlist->Add(wxBITMAP(objectgiven),wxNullBitmap);
      imlist->Add(wxBITMAP(objectreceived),wxNullBitmap);
    #endif // _BIL_ICONSIZE
  #endif // __WXMSW__
	ItemList->SetImageList(imlist);

	TreeNames=new wxArrayString;
	ResetTree();

	// Charts
	#if(wxCHECK_VERSION(3,0,0))
		#ifdef _BIL_CHART_MATLIBPLOT
			//FundChart = new wxStaticBitmap(FundPanel, wxID_ANY, wxNullBitmap, wxDefaultPosition, wxDefaultSize);
			//FundChart->Center(wxBOTH);
			//TrendChart = new wxStaticBitmap(FundPanel, wxID_ANY, wxNullBitmap, wxDefaultPosition, wxDefaultSize);
			//TrendChart->Center(wxBOTH);
		#elif defined ( _BIL_CHART_WXFREECHART )
			// NOTE (igor#1#): Workaround as wxDefaultSize is 0
			FundChart=new wxChartPanel(FundPanel,wxID_ANY,NULL,wxDefaultPosition,wxSize(100,100));
			TrendChart=new wxChartPanel(TrendPanel,wxID_ANY,NULL,wxDefaultPosition,wxSize(100,100));
		#endif // _BIL_CHART_MATLIBPLOT
	#else
		#ifdef _BIL_CHART_MATLIBPLOT

		#elif defined ( _BIL_CHART_WXFREECHART )
			FundChart=new wxChartPanel(FundPanel,wxID_ANY,NULL,wxDefaultPosition,wxDefaultSize);
			TrendChart=new wxChartPanel(TrendPanel,wxID_ANY,NULL,wxDefaultPosition,wxDefaultSize);
		#endif // _BIL_CHART_MATLIBPLOT
	#endif

	#ifdef _BIL_CHART_WXFREECHART
		FundSizer->Add(FundChart,wxSizerFlags(1).Expand());
		TrendSizer->Add(TrendChart,wxSizerFlags(1).Expand());
	#endif // _BIL_CHART_WXFREECHART

	// Right vertical part
	#if(wxCHECK_VERSION(3,0,0))
		TimePicker=new wxTimePickerCtrl(Panel2, wxID_ANY, ::wxDateTime::Now(), wxDefaultPosition, wxDefaultSize, wxTP_DEFAULT, wxDefaultValidator, wxEmptyString);
		Connect(wxEVT_TIME_CHANGED, wxDateEventHandler(BilancioMainFrame::GiveTime));
	#else
		wxString dt = MyTimeCtrl::GetCurrentTime();
		TimePicker=new MyTimeCtrl(Panel2,wxID_ANY,dt);
		TimePicker->GetCurrentTime();
	#endif
	#if(wxCHECK_VERSION(3,0,0))
		fgSizer9->Remove(1);
		fgSizer9->Insert(1,TimePicker,0,wxEXPAND|wxALL);
	#else
		fgSizer9->Insert(1,TimePicker,0,wxEXPAND|wxALL,5);
	#endif

	// Find dialog preparation
	FindData=new wxFindReplaceData(0);
	FindData->SetFlags(wxFR_DOWN);
	FindDialog = NULL;

	// Additional Event-Handlers
	Connect(wxEVT_COMMAND_FIND,wxFindDialogEventHandler(BilancioMainFrame::Find));
	Connect(wxEVT_COMMAND_FIND_NEXT,wxFindDialogEventHandler(BilancioMainFrame::Find));
	Connect(wxEVT_COMMAND_FIND_CLOSE,wxFindDialogEventHandler(BilancioMainFrame::FindClose));

	InitLanguage();
	#ifdef __WXMSW__
		SetIcon(wxICON(aaaa1bilancio_MAINICON));
	#else
		SetIcon(wxICON(Logo));
	#endif
	/*	// DON'T REMOVE - May be restored with new Product version
	MAINMENU->Enable(OldConv,ConvInstalled()); // Conversion menu item setting
	*/

	ToolBar->Show(ShowBar());
	// Wizard Check
	bool FRun=false;
	if(IsFirstRunEver()){
    FRun=true;
		wxCommandEvent evt(wxEVT_NULL,0);
		WizClick(evt);}
	RebuildTools();
	Report->SetRowMinimalAcceptableHeight(abs(Report->GetDefaultCellFont().GetPointSize())+5);
	Maximize(true);
	if(!FRun)ShowControls(false);
	Initialised=true;

	/*
	#ifdef __WXGTK__
		MainMenu_Help_->Remove(bilContributions);
	#endif
	*/

	Progress->Reparent(StatusBar);	// Necessary for the correct Progress position
	StatusBar->SetStatusWidths(2,NULL);

	this->Layout();}

BilancioMainFrame::~BilancioMainFrame(void){}

void BilancioMainFrame::InitLanguage(void){
	// Report first row initialisation
	#if(wxCHECK_VERSION(3,0,0))
		wxString inistr=wxString::Format(_("Value (%s)"),GetCurrencySymbol());
	#else
		wxString inistr=wxString::Format(_("Value (%s)"),GetCurrencySymbol().c_str());
	#endif
	Report->SetColLabelValue(3,inistr);}

void BilancioMainFrame::ResetTree(void){
	wxString inistr;
	ItemList->DeleteAllItems();
	TreeNames->Empty();
	TreeNames->Alloc(7);
	TreeNames->Add(_("Funds"),1);
	TreeNames->Add(_("Credits"),1);
	TreeNames->Add(_("Debts"),1);
	TreeNames->Add(_("Objects"),1);
	TreeNames->Add(_("Lent"),1);
	TreeNames->Add(_("Borrowed"),1);
	//TreeNames->Add(_("Shopping list"),1);
	TreeNames->Add(_("Total"),1);
	wxTreeItemId root,Obj;
	root=ItemList->AddRoot(_("Items"),-1,-1,NULL);
	Funds_tid=ItemList->AppendItem(root,TreeNames->Item(0),0,-1,NULL);
	Creds_tid=ItemList->AppendItem(root,TreeNames->Item(1),1,-1,NULL);
	Debts_tid=ItemList->AppendItem(root,TreeNames->Item(2),2,-1,NULL);
	Obj=ItemList->AppendItem(root,TreeNames->Item(3),3,-1,NULL);
	Lends_tid=ItemList->AppendItem(Obj,TreeNames->Item(4),4,-1,NULL);
	Borrowed_tid=ItemList->AppendItem(Obj,TreeNames->Item(5),5,-1,NULL);}

void BilancioMainFrame::WizClick(wxCommandEvent& event){
	wxString S,sts;
  int res = RunWizard(Progress, S, false);
	switch(res){
    case 1:
			if(!S.IsEmpty()){
				Set_DefaultDocument(S);}
		// no break statement necessary
		case 2:
			if(! S.IsEmpty()) Data->OpenDatabase(S);
			// no break statement necessary
    case 3:
      DoFirstRun();
		default:
		  ;}}

void BilancioMainFrame::ShowControls(bool V){
	Title->Show(V);
	ItemList->Show(V);
	FundPanel->Show(V);
	TrendPanel->Show(V);
	Report->Show(V);
	ShopLabel->Show(V);
	ShopList->Show(V);

	if(V){
		wxSize s;
		s=FundPanel->GetClientSize();
		FundChart->SetMinSize(s);
		FundSizer->Fit(FundChart);
		s=TrendPanel->GetClientSize();
		TrendChart->SetMinSize(s);
		TrendSizer->Fit(TrendChart);}

	Container->Layout();
	Update();}

void BilancioMainFrame::SaveClick(wxCommandEvent& event){
	if(! Data->FileData.Modified) return;
	Data->database->ReleaseSavepoint(L"rollback");
	Data->FileData.Modified = false;
	Data->database->Savepoint(L"rollback");
	UpdateMenu();
	ShowF(false);
	return;}

void BilancioMainFrame::RevertClick(wxCommandEvent& event){
	if(!Data->FileData.Modified) return;
	else if(! Data->FileData.FileName.IsEmpty()){
		wxString Msg;
		wxBell();
		Msg=::wxString::Format(_("File %s has been changed.\nDo you really want to revert to saved file?"), Data->FileData.FileView.c_str());
    if(wxMessageBox(Msg, _("Bilancio"), wxYES_NO) == wxYES){
			Data->database->Rollback(L"rollback");
			Data->ParseDatabase();

			ShowF();
			UpdateMenu();
		}}}

/*
void BilancioMainFrame::PrintClick(wxCommandEvent& event){
	wxString stsstr;
	StatusBar->SetStatusText(_("Stampa del documento in corso..."));
	::wxBeginBusyCursor(wxHOURGLASS_CURSOR);
	// Temporary bitmaps creation for charts
	//DWORD dwRetVal; // TODO -cImprovement : To be improved
	//DWORD dwBufSize=512;
	//TCHAR lpPathBuffer[512];
	wxString Ch1,Ch2;
	//UINT uRes1,uRes2;
	// ---------------------------------------
		//uRes1=GetTempFileName(lpPathBuffer,"bil",0,Ch1);
		wxString hm=::wxGetHomeDir()+L"/tmp/bil";
		Ch1=::wxFileName::CreateTempFileName(hm);
		//if(uRes1==0){
		if(Ch1.IsEmpty()){
			Error(5,wxEmptyString);
			StatusBar->SetStatusText(wxEmptyString,0);
			::wxEndBusyCursor();
			return;}
		//uRes2=GetTempFileName(lpPathBuffer,"bil",0,Ch2);
		Ch2=::wxFileName::CreateTempFileName(hm);
		//if(uRes2==0){
		if(Ch2.IsEmpty()){
			Error(5,wxEmptyString);
			StatusBar->SetStatusText(wxEmptyString,0);
			::wxEndBusyCursor();
			return;}

		wxBitmap bmp=FundChart->CopyBackbuffer();
		bmp.SaveFile(Ch1,wxBITMAP_TYPE_BMP);	// to be debugged
		bmp=TrendChart->CopyBackbuffer();
		bmp.SaveFile(Ch2,wxBITMAP_TYPE_BMP);	// to be debugged

		//Data->DocExport(true,Ch1,Ch2);
		Data->XMLExport(Ch1,Ch2);
		::wxRemoveFile(Ch1);
		::wxRemoveFile(Ch2);
	//}
	//______________________________________________
	StatusBar->SetStatusText(wxEmptyString,0);
	::wxEndBusyCursor();
	return;}
*/

/*
// TODO (igor#1#): To be restored when SQL encryption is available
void BilancioMainFrame::ModifyPasswordClick(wxCommandEvent& event){
	Data->ModPass();
	return;}
*/

void BilancioMainFrame::ExitClick(wxCommandEvent& event){
	Close(false);}

void BilancioMainFrame::FundOperation(int Op){
	if(Op==0||Op>3)return;
	int i,Ch=0;
	int search_complete = 0x00;	// equals 3 when selected fund and default fund are found
	unsigned int j;
	double Def=-1,Tot=-1,NFon,Fon=-1;
	if(Op > 1 && Data->NFun < 1){
	if(Op==2)Error(8,wxEmptyString);
	else Error(9,wxEmptyString);
	return;}
	TOPF *OPF=new TOPF(this);
	OPF->InitLabels(Op+1);
	if(Op>1){
		for(i = 0; i < Data->NFun; i++)
			if(Data->Funds[i].Name != Data->FileData.DefFund)
				OPF->Name->Append(Data->Funds[i].Name);
		if(Op==3)
			for(j=0;j<OPF->Name->GetCount();j++)
				for(int k = 0; k < Data->NFun; k++)
					if(OPF->Name->GetString(j)==Data->Funds[k].Name){
					  OPF->AddValue(FormDigits(Data->Funds[k].Value));
						break;}
		OPF->Name->Select(0);
		wxCommandEvent evt(wxEVT_NULL,0);
		if(Op == 3) OPF->NameChange(evt);}

	if(OPF->ShowModal() == wxID_OK){
		if(Op!=2)OPF->Val->GetValue().ToDouble(&NFon);
		Tot=Data->GetTot(tvFou);

		// Scan existing founds
		for(i = 0; i < Data->NFun; i++){
			if(Data->Funds[i].Name.CmpNoCase(OPF->Name->GetValue())==0){
				Fon=Data->Funds[i].Value;
				Ch=1;
				search_complete += 0x01;}
			else if(Data->Funds[i].Name.CmpNoCase(Data->FileData.DefFund)==0){
				Def=Data->Funds[i].Value;
				search_complete += 0x02;}
			if(search_complete == 0x03) break;}

		if((Fon == -1 || Tot == -1) && Op > 2){
			Error(10,wxEmptyString);
			goto FundOperationEnd;}
		switch(Op){
			case 1:
				if(Ch==1){
					Error(11,OPF->Name->GetValue());
					goto FundOperationEnd;}
				if(NFon>Def){
					Error(12,wxEmptyString);
					goto FundOperationEnd;}
				Data->SetDefaultFundValue(Def - NFon);
				if(!Data->AddValue(tvFou, -1, OPF->Name->GetValue(),NFon)){
					Error(5,wxEmptyString);
					goto FundOperationEnd;}
				break;
			case 2:
				Data->SetDefaultFundValue(Def + Fon);
				for(i = 0; i < Data->NFun; i++)
					if(Data->Funds[i].Name.CmpNoCase(OPF->Name->GetValue()) == 0){
					Data->DelValue(tvFou, Data->Funds[i].Id);
					break;}
				break;
			case 3:
				if(NFon < Fon) Data->SetDefaultFundValue(Def + Fon - NFon);
				else{
					if(Tot < (NFon - Fon)){
						Error(13, wxEmptyString);
						goto FundOperationEnd;}
					if(Def < (NFon - Fon)){
						Error(24, wxEmptyString);
						goto FundOperationEnd;}
					Data->SetDefaultFundValue(Def - NFon + Fon);}
				for(i = 0; i < Data->NFun; i++) if(Data->Funds[i].Name.CmpNoCase(OPF->Name->GetValue()) == 0){
					Data->ChangeFundValue(tvFou, Data->Funds[i].Id, NFon);
					break;}
				Data->FileData.Modified = true;}
		ShowF();}
	FundOperationEnd:
	delete OPF;
	UpdateMenu();}

void BilancioMainFrame::NewFundClick(wxCommandEvent& event){
	FundOperation(1);}

void BilancioMainFrame::RemoveFundClick(wxCommandEvent& event){
	FundOperation(2);}

void BilancioMainFrame::ResetFundClick(wxCommandEvent& event){
	FundOperation(3);}

void BilancioMainFrame::SetTotalClick(wxCommandEvent& event){
	int i,IP=-1;
	wxString curstr;
	double OldTot,Tot,Pre=-1; // OldTot: previous total value
										// Tot: new total value
										// Pre: Default fund value
	OldTot=Data->GetTot(tvFou);
	if(OldTot==-1){ // This condition should never happen
		Error(5,wxEmptyString);
		return;}
	TOPF *OPF=new TOPF(this);
	OPF->InitLabels(1);
  #ifdef __WXMSW__
    OPF->Val->SetValue(::wxString::Format("%#.2hf",OldTot));
  #else
    OPF->Val->SetValue(::wxString::Format(L"%#.2hf",OldTot));
  #endif //__WXMSW__
	if(event.GetString()==L"MainFrame"){
		OPF->btnSizer->Show(1,false);
		OPF->OKBtn->Centre(wxHORIZONTAL);}
	OPF->Layout();
	Show:
	if(OPF->ShowModal()!=wxID_OK){
		if(event.GetString()==L"MainFrame"){
			wxBell();
			goto Show;}
		else goto EndSetTotal1;}
	curstr=OPF->Val->GetValue();
	curstr.ToDouble(&Tot);
	if(OldTot>Tot){
		Error(17,wxEmptyString);
		goto EndSetTotal1;}
	for(i = 0; i < Data->NFun; i++)
		if(Data->Funds[i].Name.CmpNoCase(Data->FileData.DefFund) == 0){
		Pre=Data->Funds[i].Value;
		IP=i;
		break;}
	if(IP>-1) Data->ChangeFundValue(tvFou, Data->Funds[IP].Id, Pre + Tot - OldTot);
	else{
		wxString def=_("Default");
		Data->AddValue(tvFou, -1, def,Tot);
		Data->SetDefaultFund(def);
	}
	Data->FileData.Modified=true;
	ShowF();
	UpdateMenu();
	EndSetTotal1:
	delete OPF;}

void BilancioMainFrame::DefaultFundClick(wxCommandEvent& event){
	int i;
	TOPF *OPF=new TOPF(this);
	OPF->InitLabels(5);
	for(i = 0; i < Data->NFun; i++)
		OPF->Name->Append(Data->Funds[i].Name);
	for(i=0;i<(int)OPF->Name->GetCount();i++)if(OPF->Name->GetString(i)==Data->FileData.DefFund){
		OPF->Name->SetSelection(i);
		break;}
	if(OPF->ShowModal() == wxID_OK) Data->SetDefaultFund(OPF->Name->GetValue());
	delete OPF;}

void BilancioMainFrame::Operation(int Op){
	int i;
	wxString curstr,SelFund, MatValue;
	if(Op==0||Op>2)return;
	wxString Fund;
	double Fon=-1,Tot=-1,Val;
	wxDateTime H;
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	TOperationF *OperationF=new TOperationF(this);
	if(Op==1){
		OperationF->InitLabels(0);
		if(Data->UpdateMatters(toGain)){
			for(i=0;i<(int)Data->MattersBuffer->Count();i++)OperationF->Matter->Append(Data->MattersBuffer->Item(i));
			#if(wxCHECK_VERSION(3,0,0))
				OperationF->Matter->AutoComplete(OperationF->Matter->GetStrings());
			#endif // wxCHECK_VERSION
		}}
	else{
		OperationF->InitLabels(1);
		if(Data->UpdateMatters(toExpe)){
			for(i=0;i<(int)Data->MattersBuffer->Count();i++)OperationF->Matter->Append(Data->MattersBuffer->Item(i));
			#if(wxCHECK_VERSION(3,0,0))
				OperationF->Matter->AutoComplete(OperationF->Matter->GetStrings());
			#endif // wxCHECK_VERSION
		}}
	for(i = 0; i < Data->NFun; i++)
		OperationF->FundName->Append(Data->Funds[i].Name);
	for(i=0;i<(int)OperationF->FundName->GetCount();i++)if(OperationF->FundName->GetString(i).CmpNoCase(Data->FileData.DefFund)==0){
		OperationF->FundName->SetSelection(i);
		break;}
	#if(wxCHECK_VERSION(3,0,0))
		OperationF->Category->Append("  -");
	#else
		OperationF->Category->Append(L"  -");
	#endif // wxCHECK_VERSION
	for(int j = 0; j < Data->NCat; j++) OperationF->Category->Append(Data->Categories[j].Name);
	OperationF->Category->SetSelection(0);

	if(OperationF->ShowModal() == wxID_OK){
		curstr=OperationF->OpValue->GetValue();
		curstr.ToDouble(&Val);
		SelFund=OperationF->FundName->GetString(OperationF->FundName->GetSelection());
		for(i = 0; i < Data->NFun; i++)
			if(Data->Funds[i].Name.CmpNoCase(SelFund) == 0){
			Fund=Data->Funds[i].Name;
			Fon=Data->Funds[i].Value;
			break;}
		Tot=Data->GetTot(tvFou);
		if(Fon==-1||Tot==-1){
			Error(10,wxEmptyString);
			goto OperationEnd;}
		if(OperationF->SysTimeCheck->IsChecked())H=bilGetTime();
		else H=Data->Hour;
		switch(Op){
			case 1:
				MatValue=OperationF->Matter->GetValue();
				Data->AddOper(-1, Data->Day, H, toGain,
													#if(wxCHECK_VERSION(3, 0, 0))
														Data->bilFromCDouble(Val)
													#else
														wxString::Format(L"%f", Val)
													#endif
												, MatValue, OperationF->Category->GetSelection());
				for(i = 0; i < Data->NFun; i++)
					if(Data->Funds[i].Name == Fund){
					Data->Funds[i].Value += Val;
					Data->ChangeFundValue(tvFou, Data->Funds[i].Id, Data->Funds[i].Value);
					break;}
				break;
			case 2:
				if(Val>Fon){
					Error(13,wxEmptyString);
					goto OperationEnd;}
				MatValue=OperationF->Matter->GetValue();
				Data->AddOper(-1, Data->Day, H, toExpe,
													#if(wxCHECK_VERSION(3, 0, 0))
														Data->bilFromCDouble(Val)
													#else
														wxString::Format(L"%f", Val)
													#endif
												, MatValue, OperationF->Category->GetSelection()/* - 1*/);
				for(i = 0; i < Data->NFun; i++)
					if(Data->Funds[i].Name == Fund){
					Data->Funds[i].Value -= Val;
					Data->ChangeFundValue(tvFou, Data->Funds[i].Id, Data->Funds[i].Value);
					break;}
				if((Fon-Val==0)&&Fund!=Data->FileData.DefFund){
					wxString Msg=wxString::Format(_("Fund \"%s\" is exhausted. Do you want to delete it?"),OperationF->FundName->GetValue().c_str());
					wxBell();
					if(wxMessageBox(Msg,_("Bilancio"),wxYES_NO)==wxYES)
						for(int i = 0; i < Data->NFun; i++)
							if(Data->Funds[i].Name == Fund){
							Data->DelValue(tvFou,i);
							break;}}
				remove_obtained_shopitem(MatValue.MakeLower());}
		ShowF();}
	OperationEnd:
	delete OperationF;
	UpdateMenu();}

wxDateTime BilancioMainFrame::bilGetTime(void){
	wxDateTime now=wxDateTime::Now();
	return now;}

void BilancioMainFrame::GainClick(wxCommandEvent& event){
	Operation(1);}

void BilancioMainFrame::ExpenseClick(wxCommandEvent& event){
	Operation(2);}

void BilancioMainFrame::CategoryClick(wxCommandEvent& event){
	int Row = GetFirstSelectedRow();
	bool found;
	int i;
	if(Row < 0) return;	// Error
	if(Data->IsDate(Row))return;

	TOperationF *category = new TOperationF(this);
	category->InitLabels(2);
	#if(wxCHECK_VERSION(3,0,0))
		category->Category->Append("  -");
	#else
		category->Category->Append(L"  -");
	#endif // wxCHECK_VERSION
	for(i = 0; i < Data->NCat; i++) category->Category->Append(Data->Categories[i].Name);

	found = false;
	wxString cat = wxEmptyString;
	for(i = 0; i < Data->NCat; i++) if(Data->Categories[i].Id == Data->Lines[Row].CategoryIndex){
		cat = Data->Categories[i].Name;
		found = true;
		break;}
	if(found){
		for(int i = 1; i < (int) category->Category->GetCount(); i++) if(category->Category->GetString(i) == cat){
			category->Category->SetSelection(i);
			break;}}
	else category->Category->SetSelection(0);

	if(category->ShowModal() == wxID_OK){
		found = false;
		int cat_index = -1;
		for(i = 0; i < Data->NCat; i++) if(Data->Categories[i].Name == category->Category->GetValue()){
			cat_index = Data->Categories[i].Id;
			found = true;
			break;}
		Data->ChangeTransactionCategory(Data->Lines[Row].Id, cat_index);
		Data->FileData.Modified=true;
		ShowF(false);
		Report->SelectRow(Row,false);}
	delete category;}

void BilancioMainFrame::NewCreditClick(wxCommandEvent& event){
	int i,IDef;
	double Def=-1,Tot=-1,Tot1=-1,Val;  // Def : default fund value
																			 // Tot : total fund value
																			 // Tot1: credit total value
																			 // Val : new credit value
	wxDateTime S;
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	TSCredDeb *SCredDeb=new TSCredDeb(this);
	SCredDeb->InitLabels(false);
	if(SCredDeb->ShowModal()!=wxID_OK)goto EndNewCredit1Click;
	SCredDeb->Val->GetValue().ToDouble(&Val);
	for(i = 0; i < Data->NFun; i++)
		if(Data->Funds[i].Name.CmpNoCase(Data->FileData.DefFund)==0){
			IDef=i;
			Def=Data->Funds[i].Value;
			break;}
	Tot=Data->GetTot(tvFou);
	Tot1=Data->GetTot(tvCre);
	if((Def==-1)||(Tot==-1)||(Tot1==-1)){
		Error(18,wxEmptyString);
		goto EndNewCredit1Click;}
	if(Def<Val){
		Error(19,wxEmptyString);
		goto EndNewCredit1Click;}
	Data->AddValue(tvCre, -1, SCredDeb->Name->GetValue(), Val);
	Data->Funds[IDef].Value-=Val;
	Data->SetDefaultFundValue(Data->Funds[IDef].Value);
	if(SCredDeb->SysTimeCheck->IsChecked())S=wxDateTime(bilGetTime());
	else S=Data->Hour;
	Data->AddOper(-1, Data->Day, S, toSetCre, FormDigits(Val), SCredDeb->Name->GetValue(), -1);
	ShowF();
	UpdateMenu();
	EndNewCredit1Click:delete SCredDeb;}

void BilancioMainFrame::RemoveCreditClick(wxCommandEvent& event){
	int i,IDef=-1,IExi=-1;
	double Def=-1,Tot=-1,Fou=-1,Tot1=-1;	// Pre : default fund value
																					// Tot : fund total value
																					// Tot1: total credit value
																					// Fou : credit value
	wxString curstr;
	wxDateTime S;
	if(Data->NCre < 1){
		Error(20,wxEmptyString);
		return;}
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	TRCredDeb *RCredDeb=new TRCredDeb(this);
	RCredDeb->InitLabels(1);
	for(i = 0; i < Data->NCre; i++)RCredDeb->Name->Append(Data->Credits[i].Name);
	RCredDeb->Name->SetSelection(0);
	if(RCredDeb->ShowModal()!=wxID_OK)goto EndRemoveCredit1Click;
	if(RCredDeb->SysTimeCheck->IsChecked())S=bilGetTime();
	else S=Data->Hour;
	for(i = 0; i < Data->NFun; i++)
		if(Data->Funds[i].Name.CmpNoCase(Data->FileData.DefFund)==0){
			IDef=i;
			Def=Data->Funds[i].Value;
			break;}
	Tot=Data->GetTot(tvFou);
	for(i = 0; i < Data->NCre; i++)
		if(Data->Credits[i].Name.CmpNoCase(RCredDeb->Name->GetValue())==0){
			IExi=i;
			Fou=Data->Credits[i].Value;
			break;}
	Tot1=Data->GetTot(tvCre);
	if((Def==-1)||(Tot==-1)||(Fou==-1)||(Tot1==-1)){
		Error(18,wxEmptyString);
		goto EndRemoveCredit1Click;}
	if(RCredDeb->Opt->GetSelection()==0){
    Data->AddOper(-1, Data->Day, S, toRemCre, FormDigits(Fou), RCredDeb->Name->GetValue(), -1);
		Data->DelValue(tvCre, Data->Credits[IExi].Id);
		Data->Funds[IDef].Value += Fou;
		Data->SetDefaultFundValue(Data->Funds[IDef].Value);}
	else{
		double Par;
		curstr=RCredDeb->Val->GetValue();
		curstr.ToDouble(&Par);
		if(Par>Fou){
			Error(21, _("credit"));
			goto EndRemoveCredit1Click;}
    Data->AddOper(-1, Data->Day, S, toRemCre, FormDigits(Par), RCredDeb->Name->GetValue(), -1);
		Data->Credits[IExi].Value -= Par;
		Data->ChangeFundValue(tvCre, Data->Credits[IExi].Id, Data->Credits[IExi].Value);
		Data->Funds[IDef].Value += Par;
		Data->SetDefaultFundValue(Data->Funds[IDef].Value);}
	ShowF();
	UpdateMenu();
	EndRemoveCredit1Click:
	delete RCredDeb;}

void BilancioMainFrame::CondoneCreditClick(wxCommandEvent& event){
	int i,ICre;
	double Cre=-1,Tot=-1,Val; // Cre: Old credit value
															// Tot: Total credit value
															// Val: Value of credit to condone
	wxDateTime S;
	if(Data->NCre < 1){
		Error(20,wxEmptyString);
		return;}
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	TRCredDeb *RCredDeb=new TRCredDeb(this);
	RCredDeb->InitLabels(3);
	for(i = 0; i < Data->NCre; i++)RCredDeb->Name->Append(Data->Credits[i].Name);
	RCredDeb->Name->SetSelection(0);
	if(RCredDeb->ShowModal() == wxID_OK){
		if(RCredDeb->SysTimeCheck->IsChecked())S=bilGetTime();
		else S=Data->Hour;
		for(i = 0; i < Data->NCre; i++)
			if(Data->Credits[i].Name.CmpNoCase(RCredDeb->Name->GetValue())==0){
				ICre=i;
				Cre=Data->Credits[i].Value;
				break;}
		Tot=Data->GetTot(tvCre);
		if(Cre==-1||Tot==-1){
			Error(10,wxEmptyString);
			goto EndCondoneCredit1Click;}
		if(RCredDeb->Opt->GetSelection()==0){
			Data->AddOper(-1, Data->Day, S, toConCre, FormDigits(Cre), RCredDeb->Name->GetValue(), -1);
			Data->DelValue(tvCre, Data->Credits[ICre].Id);}
		else{
			RCredDeb->Val->GetValue().ToDouble(&Val);
			if(Val>Cre){
				Error(21, _("credit"));
				goto EndCondoneCredit1Click;}
			Data->AddOper(-1, Data->Day, S, toConCre, FormDigits(Val), RCredDeb->Name->GetValue(), -1);
			Data->Credits[ICre].Value -= Val;
			Data->ChangeFundValue(tvCre, Data->Credits[ICre].Id, Data->Credits[ICre].Value);}
		ShowF();
		UpdateMenu();
	}
	EndCondoneCredit1Click:
	delete RCredDeb;}

void BilancioMainFrame::NewDebtClick(wxCommandEvent& event){
	int i,IDef=-1;
	double Def=-1,Tot=-1,Tot1=-1,Val;  // Def : default fund value
																			 // Tot : total fund value
                                       // Tot1: debt total value
                                       // Val : new debt value
	wxString curstr;
	wxDateTime S;
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	TSCredDeb *SCredDeb=new TSCredDeb(this);
	SCredDeb->InitLabels(true);
	if(SCredDeb->ShowModal()!=wxID_OK)goto EndNewDebt1Click;
	SCredDeb->Val->GetValue().ToDouble(&Val);
	for(i = 0; i < Data->NFun; i++)
		if(Data->Funds[i].Name.CmpNoCase(Data->FileData.DefFund)==0){
			IDef=i;
			Def=Data->Funds[i].Value;
			break;}
  Tot=Data->GetTot(tvFou);
  Tot1=Data->GetTot(tvDeb);
  if((Def==-1)||(Tot==-1)||(Tot1==-1)){
		Error(18,wxEmptyString);
    goto EndNewDebt1Click;}
	Data->AddValue(tvDeb, -1, SCredDeb->Name->GetValue(),Val);
	Data->Funds[IDef].Value+=Val;
	Data->SetDefaultFundValue(Data->Funds[IDef].Value);
	if(SCredDeb->SysTimeCheck->IsChecked())S=bilGetTime();
	else S=Data->Hour;
	Data->AddOper(-1, Data->Day, S, toSetDeb, FormDigits(Val), SCredDeb->Name->GetValue(), -1);
	ShowF();
	UpdateMenu();
	EndNewDebt1Click:
	delete SCredDeb;}

void BilancioMainFrame::RemoveDebtClick(wxCommandEvent& event){
  int i,IDef=-1,IExi=-1;
	double Def=-1,Tot=-1,Fou=-1,Tot1=-1;  // Def : Default fund value
																					// Tot : Total fund value
                                          // Fou : Debt value
                                          // Tot1: Debt total value
	wxDateTime S;
	if(Data->NDeb < 1){
		Error(22,wxEmptyString);
    return;}
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	TRCredDeb *RCredDeb=new TRCredDeb(this);
	RCredDeb->InitLabels(2);
	for(i = 0; i < Data->NDeb; i++)RCredDeb->Name->Append(Data->Debts[i].Name);
	RCredDeb->Name->SetSelection(0);
	if(RCredDeb->ShowModal()!=wxID_OK)goto EndRemoveDebt1Click;
	if(RCredDeb->SysTimeCheck->IsChecked())S=bilGetTime();
	else S=Data->Hour;
	for(i = 0; i < Data->NFun; i++)
		if(Data->Funds[i].Name.CmpNoCase(Data->FileData.DefFund)==0){
		IDef=i;
		Def=Data->Funds[i].Value;
		break;}
	Tot=Data->GetTot(tvFou);
	for(i = 0; i < Data->NDeb; i++)if(Data->Debts[i].Name.CmpNoCase(RCredDeb->Name->GetValue())==0){
		IExi=i;
		Fou=Data->Debts[i].Value;
		break;}
	Tot1=Data->GetTot(tvDeb);
	if((Def==-1)||(Tot==-1)||(Fou==-1)||(Tot1==-1)){
		Error(18,wxEmptyString);
		goto EndRemoveDebt1Click;}
	if(RCredDeb->Opt->GetSelection()==0){
    if(Fou>Def){
		Error(24,wxEmptyString);
		goto EndRemoveDebt1Click;}
    Data->AddOper(-1, Data->Day, S, toRemDeb, FormDigits(Fou), RCredDeb->Name->GetValue(), -1);
		Data->DelValue(tvDeb, Data->Debts[IExi].Id);
		Data->Funds[IDef].Value -= Fou;
		Data->SetDefaultFundValue(Data->Funds[IDef].Value);}
	else{
		double Par;
		wxString curstr=RCredDeb->Val->GetValue();
		curstr.ToDouble(&Par);
		if(Par>Fou){
			Error(21, _("debit"));
			goto EndRemoveDebt1Click;}
		if(Par>Def){
			Error(24,wxEmptyString);
			goto EndRemoveDebt1Click;}
    Data->AddOper(-1, Data->Day, S, toRemDeb, FormDigits(Par), RCredDeb->Name->GetValue(), -1);
		Data->Debts[IExi].Value -= Par;
		Data->ChangeFundValue(tvDeb, Data->Debts[IExi].Id, Data->Debts[IExi].Value);
		Data->Funds[IDef].Value -= Par;
		Data->SetDefaultFundValue(Data->Funds[IDef].Value);}
	ShowF();
	UpdateMenu();
	EndRemoveDebt1Click:
	delete RCredDeb;}

void BilancioMainFrame::CondoneDebtClick(wxCommandEvent& event){
  int i,IDeb;
	double Deb=-1,Tot=-1,Val; // Deb: Old debt value
                              // Tot: Total debt value
                              // Val: Value of debt to condone
	wxDateTime S;
  if(Data->NDeb < 1){
		Error(22,wxEmptyString);
    return;}
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	TRCredDeb *RCredDeb=new TRCredDeb(this);
  RCredDeb->InitLabels(4);
	for(i = 0; i < Data->NDeb; i++)RCredDeb->Name->Append(Data->Debts[i].Name);
	RCredDeb->Name->SetSelection(0);
	if(RCredDeb->ShowModal() == wxID_OK){
		if(RCredDeb->SysTimeCheck->IsChecked())S=bilGetTime();
		else S=Data->Hour;
		for(i = 0; i < Data->NDeb; i++)if(Data->Debts[i].Name.CmpNoCase(RCredDeb->Name->GetValue())==0){
			IDeb=i;
			Deb=Data->Debts[i].Value;
			break;}
		Tot=Data->GetTot(tvDeb);
		if(Deb==-1||Tot==-1){
			Error(10,wxEmptyString);
			goto EndCondoneDebt1Click;}
		if(RCredDeb->Opt->GetSelection()==0){
			Data->AddOper(-1, Data->Day, S, toConDeb, FormDigits(Deb), RCredDeb->Name->GetValue(), -1);
			Data->DelValue(tvDeb, Data->Debts[IDeb].Id);}
		else{
			RCredDeb->Val->GetValue().ToDouble(&Val);
			if(Val>Deb){
				Error(21, _("debit"));
				goto EndCondoneDebt1Click;}
			Data->AddOper(-1, Data->Day, S, toConDeb, FormDigits(Val), RCredDeb->Name->GetValue(), -1);
			Data->Debts[IDeb].Value-=Val;
			Data->ChangeFundValue(tvDeb, Data->Debts[IDeb].Id, Data->Debts[IDeb].Value);}
		ShowF();
		UpdateMenu();}
	EndCondoneDebt1Click:
	delete RCredDeb;}

/*
void BilancioMainFrame::ContributionsClick(wxCommandEvent& event){
	TContrForm *F=new TContrForm(this);
	F->ShowModal();
	delete F;}
*/

void BilancioMainFrame::ObjectOperation(int Op){
	if(Op<1||Op>4)return;
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	wxDateTime S;
	TObjects *Objects=new TObjects(this);
	Objects->InitLabels(Op);
	if(Op<3)Objects->HideAlarm();
	if(Objects->ShowModal() == wxID_OK){
		if(Objects->SysTimeCheck->IsChecked())S=bilGetTime();
		else S=Data->Hour;
		switch(Op){
			case 1:
				Data->AddOper(-1, Data->Day, S, toGetObj, Objects->Obj->GetValue(), Objects->Per->GetValue(), -1);
				remove_obtained_shopitem(Objects->Obj->GetValue().MakeLower());
				break;
			case 2:
				Data->AddOper(-1, Data->Day, S, toGivObj, Objects->Obj->GetValue(), Objects->Per->GetValue(), -1);
				break;
			default:
				wxDateTime Alm;
				if(Objects->AlmCB->IsChecked())Alm=(Objects->Alarm->GetValue());
				else Alm=wxDateTime::Today().Add(wxDateSpan(100,0,0,0));;
				if(Op==3){
					Data->AddObject(toPre, -1, Objects->Per->GetValue(), Objects->Obj->GetValue(), Alm);
					Data->AddOper(-1, Data->Day, S, toLenObj, Objects->Obj->GetValue(), Objects->Per->GetValue(), -1);}
				else if(Op==4){
					Data->AddObject(toInP, -1, Objects->Per->GetValue(), Objects->Obj->GetValue(), Alm);
					Data->AddOper(-1, Data->Day, S, toBorObj, Objects->Obj->GetValue(), Objects->Per->GetValue(), -1);}}
		Data->FileData.Modified=true;
		ShowF(false);
		UpdateMenu();}
	delete Objects;}

void BilancioMainFrame::ReceivedClick(wxCommandEvent& event){
	ObjectOperation(1);}

void BilancioMainFrame::GiftedClick(wxCommandEvent& event){
	ObjectOperation(2);}

void BilancioMainFrame::LendClick(wxCommandEvent& event){
	ObjectOperation(3);}

void BilancioMainFrame::BorrowClick(wxCommandEvent& event){
	ObjectOperation(4);}

void BilancioMainFrame::ShopOperation(int O){
	if(O < 1 || O > 2)return;
	if(O == 2 && Data->NSho < 1){
		Error(16, wxEmptyString);
		return;}
	int i;
	TObjects *Objects = new TObjects(this);
	Objects->InitLabels(6);
	if(O == 2) Objects->HideAlarm();
	//wxDateTime today=wxDateTime::Today();
	wxDateTime Alm;
	switch(O){
		case 1:
			Objects->InitLabels(7);
			break;
		case 2:
			Objects->InitLabels(8);
			for(i = 0; i < Data->NSho; i++)Objects->Obj->Append(Data->ShopItems[i].Name);
			i = ShopList->GetSelection();
			if(i == wxNOT_FOUND) Objects->Obj->SetSelection(0);
			else Objects->Obj->SetSelection(i);}
	if(Objects->ShowModal() == wxID_OK){
		bool new_item;
		wxString item = Objects->Obj->GetValue();
		switch(O){
			case 1:
				new_item = true;
				for(i = 0; i < Data->NSho; i++) if(Data->ShopItems[i].Name.CmpNoCase(item) == 0){
					Error(48, item);
					new_item = false;
					break;}
				if(new_item){
					if(Objects->AlmCB->IsChecked()) Alm = Objects->Alarm->GetValue();
					else Alm = wxDateTime::Today().Add(wxDateSpan(100, 0, 0, 0));
					Data->AddShopItem(-1, item, Alm);}
				break;
			case 2:
				for(i = 0; i < Data->NSho; i++)
				if(Data->ShopItems[i].Name.CmpNoCase(item) == 0){
					Data->DelShopItem(Data->ShopItems[i].Id);
					break;}}
		ShowF(false);}
	//EndShopOperation:
	delete Objects;
	UpdateMenu();}

void BilancioMainFrame::AddShopItemClick(wxCommandEvent& event){
	ShopOperation(1);}

void BilancioMainFrame::DelShopItemClick(wxCommandEvent& event){
	ShopOperation(2);}

void BilancioMainFrame::GetBackClick(wxCommandEvent& event){
	ObjectOperation2(1);}

void BilancioMainFrame::GiveBackClick(wxCommandEvent& event){
	ObjectOperation2(2);}

void BilancioMainFrame::ObjectOperation2(int Op){
	int i;
	if(Op<1||Op>2)return;
	bool Ex;
	wxDateTime S;
	switch(Op){
		case 1:
			if(Data->NLen == 0){
				Error(14,wxEmptyString);
				return;}
			break;
		case 2:
			if(Data->NBor == 0){
				Error(15,wxEmptyString);
				return;}
			break;
		default:
			return;}
	#if(! wxCHECK_VERSION(3,0,0))
		GiveTime();
	#endif
	TGetBackF *GetBackF=new TGetBackF(this);
	if(Op==1)GetBackF->InitLabels(false);
	else GetBackF->InitLabels(true);
	switch(Op){
		case 1:
			for(i = 0; i < Data->NLen; i++){
				Ex=false;
				for(unsigned int j=0;j<GetBackF->Per->GetCount();j++)if(GetBackF->Per->GetString(j)==Data->Lent[i].Name){
					Ex=true;
					break;}
				if(!Ex)GetBackF->Per->Append(Data->Lent[i].Name);
				GetBackF->Memo->Add(Data->Lent[i].Name+L"|"+Data->Lent[i].Object,1);}
			break;
		case 2:
			for(i = 0; i < Data->NBor; i++){
				Ex=false;
				for(unsigned int j=0;j<GetBackF->Per->GetCount();j++)if(GetBackF->Per->GetString(j)==Data->Borrowed[i].Name){
					Ex=true;
					break;}
				if(!Ex)GetBackF->Per->Append(Data->Borrowed[i].Name);
				GetBackF->Memo->Add(Data->Borrowed[i].Name+L"|"+Data->Borrowed[i].Object);}}
	GetBackF->Per->SetSelection(0);
	wxCommandEvent evt(wxEVT_NULL,0);
	GetBackF->PerChange(evt);
	if(GetBackF->ShowModal() == wxID_OK){
		if(GetBackF->SysTimeCheck->IsChecked())S=bilGetTime();
		else S=Data->Hour;
		switch(Op){
			case 1:
				for(i = 0; i < Data->NLen; i++)if(Data->Lent[i].Name == GetBackF->Per->GetValue() && Data->Lent[i].Object == GetBackF->Obj->GetValue()){
						Data->DelObject(toPre, Data->Lent[i].Id);
						break;}
				Data->AddOper(-1, Data->Day, S, toBakObj, GetBackF->Obj->GetValue(), GetBackF->Per->GetValue(), -1);
				break;
			case 2:
				for(i = 0; i < Data->NBor; i++)
					if(Data->Borrowed[i].Name == GetBackF->Per->GetValue() && Data->Borrowed[i].Object == GetBackF->Obj->GetValue()){
						Data->DelObject(toInP, Data->Borrowed[i].Id);
						break;}
				Data->AddOper(-1, Data->Day, S, toRetObj, GetBackF->Obj->GetValue(), GetBackF->Per->GetValue(), -1);}
		Data->FileData.Modified=true;
		ShowF(false);}
	delete GetBackF;}

void BilancioMainFrame::ReportSelect(wxGridEvent& event){
  Report->SelectRow(event.GetRow());}

void BilancioMainFrame::ReportRangeSelect(wxGridRangeSelectEvent& event){
  wxGridCellCoordsArray selTop=Report->GetSelectionBlockTopLeft();
  wxGridCellCoordsArray selBottom=Report->GetSelectionBlockBottomRight();
  for(unsigned int i=0;i<selTop.Count();i++ )Report->ClearSelection();}

int BilancioMainFrame::GetFirstSelectedRow(void){
	wxArrayInt rows=Report->GetSelectedRows();
	if(rows.GetCount()<=0){
		// Error
		return -1;
	}
	return rows.Item(0);
}

void BilancioMainFrame::CopyClick(wxCommandEvent& event){
	int r=GetFirstSelectedRow();
	if(r>=0){
		//int I=Report->GetSelectedRows()[0]-1;
		double cur;
		// Date copy
    #ifdef __WXMSW__
		wxString Clip=Data->Lines[r].Date.Format("%d/%m/%Y",wxDateTime::Local)+L" ";
    #else
		wxString Clip=Data->Lines[r].Date.Format(L"%d/%m/%Y",wxDateTime::Local)+L" ";
    #endif // __WXMSW__
		wxArrayString *CB=new wxArrayString();
		CB->Clear();
		if(Data->Lines[r].IsDate){ // The line contains a date record
			CB->Add(Clip);
			// Total Copy
			Clip=GetCurrencySymbol();
			Clip+=char(32);
			Data->Lines[r].Value.ToDouble(&cur);
			Clip+=FormDigits(cur);
			Clip=Clip+L" ("+GetLogString(0)+L")";
			CB->Add(Clip);}
		else{ // The line contains an operation
			// Time copy
      #ifdef __WXMSW__
        Clip=Clip+Data->Lines[r].Time.Format("%H:%M",wxDateTime::Local)+" ";
      #else
        Clip=Clip+Data->Lines[r].Time.Format(L"%H:%M",wxDateTime::Local)+L" ";
      #endif // __WXMSW__
			// Operation type copy
			Clip=Clip+GetLogString(Data->Lines[r].Type)+L": ";
			// Value copy
			if(Data->Lines[r].Type<9){
				Data->Lines[r].Value.ToDouble(&cur);
				Clip=Clip+GetCurrencySymbol()+L" "+FormDigits(cur);}
			else Clip=Clip+Data->Lines[r].Value;
			// Reason copy
			Clip=Clip+L" "+Data->Lines[r].Reason;
			CB->Add(Clip);}
		// Category copy
		if(Data->Lines[r].CategoryIndex != -1)
			for(int k = 0; k < Data->NCat; k++)if(Data->Categories[k].Id == Data->Lines[r].CategoryIndex){
				CB->Add(Data->Categories[k].Name);
				break;}
		Clip=wxEmptyString;
		for(unsigned int z=0;z<=CB->Count()-1;z++){
      if(z>0)Clip+=(L"\n");
      Clip+=CB->Item(z);}
		wxTextDataObject *text=new wxTextDataObject();
		text->SetText(Clip);
		wxTheClipboard->Open();
    wxTheClipboard->SetData(text);
		wxTheClipboard->Flush();
		wxTheClipboard->Close();
		//delete text;	// NOTE (igor#1#): Do not delete data owned by the clipboard, it would make te app crash!
		}}

void BilancioMainFrame::TextConvClick(wxCommandEvent& event){
	StatusBar->SetStatusText(_("Document conversion i progress..."),0);

	wxString Ch1,Ch2;
  wxString tmp_file = GetTempFile();
	Ch1=::wxFileName::CreateTempFileName(tmp_file);

	if(Ch1.IsEmpty()){
		Error(44,wxEmptyString);
		StatusBar->SetStatusText(wxEmptyString,0);
		::wxEndBusyCursor();
		return;}
	Ch2=::wxFileName::CreateTempFileName(tmp_file);
	if(Ch2.IsEmpty()){
		Error(44,wxEmptyString);
		StatusBar->SetStatusText(wxEmptyString,0);
		::wxEndBusyCursor();
		return;}

	wxBitmap bmp;
	#ifdef _BIL_CHART_MATLIBPLOT
		//bmp = FundChart->GetBitmap();
		bmp = Fund_bmp;
	#elif defined ( _BIL_CHART_WXFREECHART )
		bmp=FundChart->CopyBackbuffer();
	#endif // _BIL_CHART_MATLIBPLOT
	bmp.SaveFile(Ch1, wxBITMAP_TYPE_PNG);

	#ifdef _BIL_CHART_MATLIBPLOT
		//bmp = TrendChart->GetBitmap();
		bmp = Trend_bmp;
	#elif defined ( _BIL_CHART_WXFREECHART )
		bmp=TrendChart->CopyBackbuffer();
	#endif // _BIL_CHART_MATLIBPLOT
	bmp.SaveFile(Ch2, wxBITMAP_TYPE_PNG);

	if(Master_Attached){
		//int min_date, max_date;
		//wxDateTime cal_date, min_datetime, max_datetime;
		wxDateTime cal_date = Calendar->GetDate();
		int month = cal_date.GetMonth();
		int year = cal_date.GetYear();

		if(((month >= Data->FileData.Month - 1) && (year == Data->FileData.Year)) || (year > Data->FileData.Year))
			Data->XMLExport(Ch1, Ch2);
		else Data->XMLExport_archive(Ch1, Ch2, cal_date);
	}
	else Data->XMLExport(Ch1, Ch2);

	// Remove temporary chart files
	::wxRemoveFile(Ch1);
	::wxRemoveFile(Ch2);

	StatusBar->SetStatusText(wxEmptyString,0);}

#if _BIL_INSTALLEDCONV == 1
	void BilancioMainFrame::OldConv1Click(wxCommandEvent& event){
		wxFileDialog *Dialog=new wxFileDialog(wxTheApp->GetTopWindow(),wxEmptyString,wxEmptyString,wxEmptyString,wxEmptyString,wxFD_OPEN,wxDefaultPosition,wxDefaultSize,L"Dialog");
		Dialog->SetDirectory(GetOSDocDir());
		wxString filter = _("Bilancio documents");
		filter += L"|*.bil;*.BIL";
		Dialog->SetWildcard(filter);
		if(Dialog->ShowModal() != wxID_OK){
			delete Dialog;
			return;}

		wxString File = Dialog->GetPath();
		if(CheckFileFormat(File) == bilFFORMAT_31){
			delete Dialog;
			return;}
		wxString Proc;
		#ifdef __WXMSW__
			Proc="\""+GetInstallationPath()+L"\\bilconvert.exe\"";
			wxExecute(Proc, wxEXEC_SYNC | wxEXEC_SHOW_CONSOLE);
		#else
			Proc = Console + wxGetCwd() + L"/bilconvert \"" + File + L"\"";
			wxExecute(Proc, wxEXEC_SYNC);
		#endif

		delete Dialog;}
#endif // _BIL_INSTALLEDCONV

void BilancioMainFrame::OptionsClick(wxCommandEvent& event){
	if(ShowOptionsDialog()){
		wxString label;
		ToolBar->Show(ShowBar());
		::wxBeginBusyCursor(wxHOURGLASS_CURSOR);
		StatusBar->SetStatusText(_("Field update in progress..."));
		RebuildTools();
    ShowF();
		StatusBar->SetStatusText(wxEmptyString);
		::wxEndBusyCursor();}}

void BilancioMainFrame::RebuildTools(void){
	int i;
	unsigned int j;
	#ifndef __OPENSUSE__
		wxMenuItemList::Node* node;
	#else
		wxMenuItemList::compatibility_iterator node;
	#endif // __OPENSUSE__
	wxMenuItem *item;
	wxMenuItemList sublist=MainMenu_ExtTools->GetMenuItems();
	for(j=0;j<sublist.GetCount();j++){
		node=sublist.Item(j);
		item=node->GetData();
		MainMenu_ExtTools->Delete(item);}
	for(i = 0; i < GetToolCount(); i++){
		item=new wxMenuItem(MainMenu_ExtTools, bilExtTool + i, GetTool(i), wxEmptyString, wxITEM_NORMAL);
		Connect(bilExtTool+i,wxEVT_COMMAND_MENU_SELECTED,wxCommandEventHandler(BilancioMainFrame::ExtClick));
		MainMenu_ExtTools->Append(item);}}

void BilancioMainFrame::ShowF(bool doChart){
	int i;
	Title->SetLabel(CreateTitle(true));
	// TreeList construction
	ItemList->DeleteAllItems();
	ResetTree();
	for(i = 0; i < Data->NFun; i++)
		AddNode(Funds_tid, Data->Funds[i].Name, FormDigits(Data->Funds[i].Value));
	for(i = 0; i < Data->NCre; i++) AddNode(Creds_tid, Data->Credits[i].Name, FormDigits(Data->Credits[i].Value));
	for(i = 0; i < Data->NDeb; i++) AddNode(Debts_tid, Data->Debts[i].Name, FormDigits(Data->Debts[i].Value));
	for(i = 0; i < Data->NLen; i++) AddNode(Lends_tid, Data->Lent[i].Name, Data->Lent[i].Object);
	for(i=  0; i < Data->NBor; i++) AddNode(Borrowed_tid, Data->Borrowed[i].Name, Data->Borrowed[i].Object);

	// Add totals to TreeList
	AddNode(Funds_tid,TreeNames->Item(6),FormDigits(Data->GetTot(tvFou)));
	AddNode(Creds_tid,TreeNames->Item(6),FormDigits(Data->GetTot(tvCre)));
	AddNode(Debts_tid,TreeNames->Item(6),FormDigits(Data->GetTot(tvDeb)));

	// show report
	if(Report->GetNumberRows()>0)Report->DeleteRows(0,Report->GetNumberRows(),false);
	Report->SetDefaultCellTextColour(backup_report_colour);

	Report->AppendRows(Data->NLin, false);

	wxFont font = Report->GetCellFont(0, 0);
	#if( wxCHECK_VERSION(3,0,0) )
		font.MakeBold();
	#endif

	for(i = 0; i < Data->NLin; i++){
		if(Data->Lines[i].IsDate){
			Report->SetCellValue(i,0,Data->Lines[i].Date.FormatDate());
			Report->SetCellFont(i, 0, font);
			Report->SetCellValue(i,2,GetLogString(0));
			Report->SetCellFont(i, 2, font);
			Report->SetCellFont(i, 3, font);}
		else{
			Report->SetCellValue(i,1,Data->Lines[i].Time.Format(L"%H:%M",wxDateTime::Local));
			Report->SetCellValue(i,2,GetLogString(Data->Lines[i].Type));
			Report->SetCellValue(i,4,Data->Lines[i].Reason);
			if(Data->Lines[i].CategoryIndex != -1)
				for(int j = 0; j < Data->NCat; j++) if(Data->Categories[j].Id == Data->Lines[i].CategoryIndex){
					Report->SetCellValue(i, 5, Data->Categories[j].Name);
					break;}}
		if(Data->Lines[i].Type<9){
			double cur;
			Data->Lines[i].Value.ToDouble(&cur);
			Report->SetCellValue(i,3,FormDigits(cur));
			if(Report->GetCellValue(i,3).IsEmpty())
				Report->SetCellValue(i,3,_("Zero"));}
		else Report->SetCellValue(i,3,Data->Lines[i].Value);
		Report->SetCellAlignment(i,3,wxALIGN_RIGHT,wxALIGN_CENTRE);}

	Report->AutoSizeColumns(false);
	Report->SelectRow(Report->GetNumberRows()-1,false);
	Report->MakeCellVisible(Data->NLin - 1, 0);

	if(doChart){
		TrendChart->Show(ShowTrendChart());
		FundChart->Show(ShowFundChart());
		if(TrendChart->IsShown()||FundChart->IsShown())PaintChart();
	}

	// Shopping list
	int thisYear, alarmYear;	// current year and year of item alarm
	wxString tmp_str;					// str to be appended in Shopping List view
	wxDateTime alarmCalendar;	// alarm from the Shop item
	thisYear = ::wxDateTime::Today().GetYear();

	ShopList->Clear();

	// Determine maximum string length in the list;
	int len, max_len = 0;
	for(i = 0; i < Data->NSho; i++){
		len = Data->ShopItems[i].Name.length();
		if(len > max_len) max_len = len;}

	for(i = 0; i < Data->NSho; i++){
		tmp_str = Data->ShopItems[i].Name;

		// TODO (igor#1#): Improvement: tab alignment in the list view
		len = ((max_len - tmp_str.length()) / 4) + 2;
		if(len > 1) len ++;
		#if(wxCHECK_VERSION(3,0,0))
			for(int j = 0; j < len; j++) tmp_str += "\t";
			tmp_str += "- ";
		#else
			for(int j = 0; j < len; j++) tmp_str += L"\t";
			tmp_str += L"- ";
		#endif

		alarmCalendar = Data->ShopItems[i].Alarm;
		alarmYear = alarmCalendar.GetYear();
		if((alarmYear - thisYear) < 80) tmp_str += alarmCalendar.FormatDate();
		else tmp_str += _("No alarm");
		ShopList->Append(tmp_str);}
	Update();}

/*
void BilancioMainFrame::AddNode(wxString Node,wxString OptNode,wxString Name,wxString Value){
	for(int i=0;i<ItemList->GetCount();i++)if(ItemList->GetString(i)==Node){
		if(Node==TreeNames->Item(3)){
			bool E=false; // true name for objects already exists
			int P;
			for(int j=0;j<ItemList->Items->Item[i]->Count;j++)if(ItemList->Items->Item[i]->Item[j]->Text==OptNode){
				for(int k=0;k<ItemList->Items->Item[i]->Item[j]->Count;k++)if(ItemList->Items->Item[i]->Item[j]->Item[k]->Text==Name){
					P=k;E=true;break;}
				if(E){
					ItemList->Items->AddChild(ItemList->Items->Item[i]->Item[j]->Item[P],Value);
					ItemList->Items->Item[i]->Item[j]->Item[P]->Item[ItemList->Items->Item[i]->Item[j]->Item[P]->Count-1]->ImageIndex=-1;
					ItemList->Items->Item[i]->Item[j]->Item[P]->Item[ItemList->Items->Item[i]->Item[j]->Item[P]->Count-1]->SelectedIndex=-1;
					ItemList->Items->Item[i]->Item[j]->Item[P]->Item[ItemList->Items->Item[i]->Item[j]->Item[P]->Count-1]->StateIndex=-1;}
				else{
					int Ind;
					if(OptNode==TreeNames->Strings[4])Ind=16;
					else if(OptNode==TreeNames->Strings[5])Ind=17;
					else Ind=-1;
					ItemList->Items->AddChildFirst(ItemList->Items->Item[i]->Item[j],Name);
					ItemList->Items->Item[i]->Item[j]->Item[0]->ImageIndex=Ind;
			ItemList->Items->Item[i]->Item[j]->Item[0]->SelectedIndex=Ind;
					ItemList->Items->Item[i]->Item[j]->Item[0]->StateIndex=Ind;
					ItemList->Items->AddChild(ItemList->Items->Item[i]->Item[j]->Item[0],Value);
					ItemList->Items->Item[i]->Item[j]->Item[0]->Item[ItemList->Items->Item[i]->Item[j]->Item[0]->Count-1]->ImageIndex=-1;
					ItemList->Items->Item[i]->Item[j]->Item[0]->Item[ItemList->Items->Item[i]->Item[j]->Item[0]->Count-1]->SelectedIndex=-1;
					ItemList->Items->Item[i]->Item[j]->Item[0]->Item[ItemList->Items->Item[i]->Item[j]->Item[0]->Count-1]->StateIndex=-1;}}}
		else{
			int Ind,A;
			if(Node==TreeNames->Strings[0])Ind=6;
			else if(Node==TreeNames->Strings[1])Ind=18;
			else if(Node==TreeNames->Strings[2])Ind=21;
			else if(Node==TreeNames->Strings[6])Ind=16;
			if(Node!=TreeNames->Strings[7]){
				ItemList->Items->AddChildFirst(ItemList->Items->Item[i],Name);
				A=0;}
			else{
				ItemList->Items->AddChild(ItemList->Items->Item[i],Name);
				A=ItemList->Items->Item[i]->Count-1;}
			ItemList->Items->Item[i]->Item[A]->ImageIndex=Ind;
			ItemList->Items->Item[i]->Item[A]->SelectedIndex=Ind;
			ItemList->Items->Item[i]->Item[A]->StateIndex=Ind;
			if(Node!=TreeNames->Strings[6]){
				Ind=-1;
				ItemList->Items->AddChild(ItemList->Items->Item[i]->Item[A],Value);
				ItemList->Items->Item[i]->Item[A]->Item[ItemList->Items->Item[i]->Item[A]->Count-1]->ImageIndex=Ind;
				ItemList->Items->Item[i]->Item[A]->Item[ItemList->Items->Item[i]->Item[A]->Count-1]->SelectedIndex=Ind;
				ItemList->Items->Item[i]->Item[A]->Item[ItemList->Items->Item[i]->Item[A]->Count-1]->StateIndex=Ind;}}}
}
*/

void BilancioMainFrame::AddNode(wxTreeItemId Node,/*wxString OptNode,*/wxString Name,wxString Value){
  int index=-1;
  wxTreeItemId item;
  if(Node==Funds_tid)index=0;
  else if(Node==Creds_tid)index=1;
  else if(Node==Debts_tid)index=2;
  else if(Node==Lends_tid)index=4;
  else if(Node==Borrowed_tid)index=5;
  item=ItemList->AppendItem(Node,Name,index,index,NULL);
  ItemList->AppendItem(item,Value,index,index,NULL);
}

void BilancioMainFrame::PaintChart(void){
	int i;
	if(ShowFundChart()){

		#ifdef _BIL_CHART_MATLIBPLOT
			double sum_small_funds = 0;
			bool default_is_small = false;

		  wxString tmp_file, data_fname, chart_fname, line;
		  wxString Proc = L"python " + GetDataDir();
		  Proc += L"pie.py ";
		  Proc += L"/usr/share/locale ";	// TODO (igor#1#): Check if it is possible to get this path programmatically

			tmp_file = GetTempFile();
			data_fname = ::wxFileName::CreateTempFileName(tmp_file);
			wxTextFile *file = new wxTextFile(data_fname);
			file->Create();

			for(i = 0; i < Data->NFun; i++){
				if((Data->Funds[i].Value / Data->Tot_Funds) > 0.01){
					#if( wxCHECK_VERSION(3,0,0) )
						line = Data->Funds[i].Name + ", " + wxString::FromCDouble(Data->Funds[i].Value, 2) + ", 0";
					#else
						line = Data->Funds[i].Name + L", " + FormDigits(Data->Funds[i].Value) + L", 0";
					#endif
					if(Data->Funds[i].Name.Cmp(Data->FileData.DefFund) == 0) line += L".1";
					file->AddLine(line);}
				else{
					sum_small_funds += Data->Funds[i].Value;
					if(Data->Funds[i].Name.Cmp(Data->FileData.DefFund) == 0) default_is_small = true;}
			}
			if(sum_small_funds){
				#if( wxCHECK_VERSION(3,0,0) )
					line = _("Others") + ", " + wxString::FromCDouble(sum_small_funds, 2) + ", 0";
				#else
					line = _("Others");
					line += L", ";
					line += FormDigits(sum_small_funds);
					line += L", 0";
				#endif

				if(default_is_small) line += L".1";
				file->AddLine(line);
			}

			file->Write(wxTextFileType_Unix,wxConvUTF8);
			// file closure
			file->Close();
			delete file;

			Proc += data_fname + L" ";

			chart_fname = ::wxFileName::CreateTempFileName(tmp_file);
			::wxRemoveFile(chart_fname);
			chart_fname += L".png";
			Proc += chart_fname;

			wxExecute(Proc, wxEXEC_SYNC);
			::wxRemoveFile(data_fname);

			FundChart->SetBitmap(GetScaledBitmap(chart_fname, FundChart));

			::wxRemoveFile(chart_fname);

		#elif defined ( _BIL_CHART_WXFREECHART )
			// Fund Chart construction
			int cindex=0;	// colour array index
			PiePlot *FundPlot=new PiePlot();
			#ifndef __FEDORA__
				FundPlot->Set3DView(true);
				FundPlot->SetEllipticAspect(.6);
			#endif // __FEDORA__
			double fundvalues[Data->NFun];
			wxString fundnames[Data->NFun];
			wxColour colours[Data->NFun];
			for(i = 0; i < Data->NFun; i++){
				fundvalues[i]=Data->Funds[i].Value;
				fundnames[i]=Data->Funds[i].Name;}
			CategorySimpleDataset* dataset = new CategorySimpleDataset(fundnames, WXSIZEOF(fundnames));
			dataset->AddSerie(_("Funds"),fundvalues, WXSIZEOF(fundvalues));
			// colour array initialization
			i=0;
			while(cindex < Data->NFun){
				switch(i){
					case 1:
					colours[cindex]=wxColour(0xFF,0xFF, 0x99);
					break;
					case 2:
					colours[cindex]=wxColour(0x3D, 0xEB, 0x3D);
					break;
					case 3:
					colours[cindex]=wxColour(0xFF, 0x00, 0x00);
					break;
					case 4:
					colours[cindex]=wxColour(0x00, 0xFF, 0x00);
					break;
					case 5:
					colours[cindex]=wxColour(0x00, 0x00, 0xFF);
					break;
					default:
					colours[cindex]=wxColour(0x99, 0xCC, 0xFF);
					i=0;}
				i++;
				cindex++;}
			ColorScheme* colorScheme = new ColorScheme(colours, WXSIZEOF(colours)); FundPlot->SetDataset(dataset);
			FundPlot->SetColorScheme(colorScheme);
			FundChart->SetChart(new Chart(FundPlot, _("Fund Chart")));
		#endif // _BIL_CHART_MATLIBPLOT
	}
	if(ShowTrendChart()){
		// Trend chart construction
		wxDateTime::Month m;
		switch(Data->FileData.Month){
			case 1:
				m = wxDateTime::Jan;
				break;
			case 2:
				m = wxDateTime::Feb;
				break;
			case 3:
				m = wxDateTime::Mar;
				break;
			case 4:
				m = wxDateTime::Apr;
				break;
			case 5:
				m = wxDateTime::May;
				break;
			case 6:
				m = wxDateTime::Jun;
				break;
			case 7:
				m = wxDateTime::Jul;
				break;
			case 8:
				m = wxDateTime::Aug;
				break;
			case 9:
				m = wxDateTime::Sep;
				break;
			case 10:
				m = wxDateTime::Oct;
				break;
			case 11:
				m = wxDateTime::Nov;
				break;
			case 12:
				m = wxDateTime::Dec;
				break;
			default:
				m = wxDateTime::Inv_Month;}

		int number_of_days = wxDateTime::GetNumberOfDays(m, Data->FileData.Year, wxDateTime::Gregorian);

		double trendvalues[number_of_days];
		for(i = 0; i < number_of_days; i++) trendvalues[i] = -1;
		const wxChar *trenddates[number_of_days];
		time_t times[WXSIZEOF(trenddates)];
    wxDateTime LastDate = wxDateTime::Today(), dt;
    double	LastValue = 0
    				#ifdef _BIL_CHART_WXFREECHART
							,
							MinValue = 0,
							MaxValue = 0
						#endif // _BIL_CHART_WXFREECHART
						;

		int j = 0;
		for(i = 0; i < Data->NLin; i++){
			if(Data->IsDate(i)){
				j = Data->Lines[i].Date.GetDay() - 1;
				LastDate = Data->Lines[i].Date;
				Data->Lines[i].Value.ToDouble(&LastValue);
				trendvalues[j]=LastValue;
				times[j]=LastDate.GetTicks();}}

		bool	found_before = false,
					found_after;
		int k;
		for(i = 0; i < number_of_days; i++){
			if(trendvalues[i] == -1){
				if(found_before){
					found_after = false;
					for(k = i+1; k < number_of_days; k++)
						if(trendvalues[k] != -1){
							found_after = true;
							break;}

					if(found_after) trendvalues[i] = trendvalues[i-1];
					else trendvalues[i] = 0;}
				else trendvalues[i] = 0;
				dt = wxDateTime(i+1, m, Data->FileData.Year, 0, 0, 0, 0);
				times[i] = dt.GetTicks();}
			else found_before = true;}

		#ifdef _BIL_CHART_MATLIBPLOT
		  wxString tmp_file, data_fname, chart_fname, line;
		  wxString Proc = L"python " + GetDataDir();
		  Proc += L"trend.py ";
		  Proc += L"/usr/share/locale ";	// TODO (igor#1#): Check if it is possible to get this path programmatically

			tmp_file = GetTempFile();
			data_fname = ::wxFileName::CreateTempFileName(tmp_file);
			wxTextFile *file = new wxTextFile(data_fname);
			file->Create();

			for(i = 0; i < number_of_days; i++)
				#if( wxCHECK_VERSION(3,0,0) )
					file->AddLine(wxString::Format("%d", int(times[i])) + ", " + wxString::FromCDouble(trendvalues[i], 2));
				#else
					file->AddLine(wxString::Format(L"%d", int(times[i])) + L", " + FormDigits(trendvalues[i]));
				#endif

			file->Write(wxTextFileType_Unix,wxConvUTF8);
			// file closure
			file->Close();
			delete file;

			Proc += data_fname + L" ";

			chart_fname = ::wxFileName::CreateTempFileName(tmp_file);
			::wxRemoveFile(chart_fname);
			chart_fname += L".png";
			Proc += chart_fname;

			if(! Data->FileData.ReadOnly){
				wxChar color;
				double currenttotal = Data->GetTot(tvFou);
				if(LastValue == currenttotal) color = 'c';
				else if(LastValue < currenttotal) color = 'g';
				else color = 'r';

				#if( wxCHECK_VERSION(3,0,0) )
					Proc += " " + wxString(color) + " " + wxString::FromCDouble(currenttotal, 2);
				#else
					Proc += L" " + wxString(color) + L" " + FormDigits(currenttotal);
				#endif
			}

			wxExecute(Proc, wxEXEC_SYNC);
			::wxRemoveFile(data_fname);

			TrendChart->SetBitmap(GetScaledBitmap(chart_fname, TrendChart, true));

			::wxRemoveFile(chart_fname);

		#elif defined ( _BIL_CHART_WXFREECHART )

			XYPlot *trendplot = new XYPlot();
			wxColour color_trend = wxColour(0, 0, 0);
			XYLineRenderer *renderer_trend=new XYLineRenderer();
			renderer_trend->SetSerieColour(0,&color_trend);
			TimeSeriesDataset *dataset1 = new TimeSeriesDataset(trendvalues, times, WXSIZEOF(trendvalues));
			dataset1->SetRenderer(renderer_trend);
			trendplot->AddDataset(dataset1);

			if(!Data->FileData.ReadOnly){
				// trend addition
				double trend_trend_values[/*j*/number_of_days+1];
				const wxChar *trend_trend_dates[/*j*/number_of_days+1];
				time_t times_trend[WXSIZEOF(trend_trend_dates)];
				for(i=0; i<= j; i++){
					trend_trend_values[i] = trendvalues[i];
					times_trend[i] = times[i];}
				double currenttotal=Data->GetTot(tvFou);
				trend_trend_values[j+1] = currenttotal;
				times_trend[j+1] = LastDate.GetTicks();			// set during main Trend construction loop
				dt = wxDateTime::Today();
				for(i = j+2; i < number_of_days; i++){
					trend_trend_values[i] = currenttotal;
					times_trend[i] = dt.GetTicks();}
				wxColour color_trend_trend;
				if(LastValue==currenttotal)color_trend_trend=wxColour(255,255,0);
				else if(LastValue<currenttotal)color_trend_trend=wxColour(0,100,0);
				else color_trend_trend=wxColour(255,0,0);

				renderer_trend->SetSerieColour(0,&color_trend_trend);
				TimeSeriesDataset *dataset2 = new TimeSeriesDataset(trend_trend_values, times_trend, WXSIZEOF(trend_trend_values));
				dataset2->SetRenderer(renderer_trend);
				trendplot->AddDataset(dataset2);

				if(trend_trend_values[j] < MinValue) MinValue = trend_trend_values[j];
				if(trend_trend_values[j] > MaxValue) MaxValue = trend_trend_values[j];}

			NumberAxis *leftAxis = new NumberAxis(AXIS_LEFT);
			DateAxis *bottomAxis = new DateAxis(AXIS_BOTTOM);
			bottomAxis->SetVerticalLabelText(true);
			bottomAxis->SetDateFormat(L"%d/%m");

			// Set y scale
			if(MinValue == 0){
				for(i = 0; i < j; i++)if(trendvalues[i] > 0){
					MinValue = trendvalues[i];
					break;}}
			if(MaxValue == 0) MaxValue = trendvalues[0];
			for(i=1;i<j;i++){
				if((trendvalues[i] != -1) && (trendvalues[i] < MinValue)) MinValue=trendvalues[i];
				if(trendvalues[i]>MaxValue)MaxValue=trendvalues[i];}
			int p=0,num;
			double res=MinValue;
			while(res>=1){
				res=res/10;
				if(res>=1)p++;}
			p=pow(10,p);
			num=MinValue/p;
			if(num > 1) num--;
			MinValue=num*p;
			p=0;
			double real_max_value = res = MaxValue;
			while(res >= 1){
				res = res / 10;
				if(res >= 1) p++;}
			p = pow(10, p);
			num = MaxValue / p + 1;
			MaxValue = num * p;

			if((p > 1000) && (MaxValue / MinValue > 2)) {
				double test_max_value = MaxValue - (MinValue / 4 * 3);
				if(test_max_value >= real_max_value) MaxValue = test_max_value;
				else {
					test_max_value = MaxValue - (MinValue / 2);
					if(test_max_value >= real_max_value) MaxValue = test_max_value;}
			}

			leftAxis->SetFixedBounds(MinValue, MaxValue);

			trendplot->AddAxis(leftAxis);
			trendplot->AddAxis(bottomAxis);
			trendplot->LinkDataVerticalAxis(0, 0);
			trendplot->LinkDataHorizontalAxis(0, 0);
			if(! Data->FileData.ReadOnly){
				trendplot->LinkDataVerticalAxis(1, 0);
				trendplot->LinkDataHorizontalAxis(1, 0);}
			TrendChart->SetChart(new Chart(trendplot, _("Trend Chart")));

		#endif // _BIL_CHART_MATLIBPLOT
	}
	FreshChartShown = true;
}

void BilancioMainFrame::HelpMenuClick(wxCommandEvent& event){
  wxString Page;
  #ifdef __WXMSW__
    Page=GetInstallationPath();
    if(Lan==wxLANGUAGE_ITALIAN)Page+=L"\\bilancio-msw_it.html";
    else Page+=L"\\bilancio-msw_en.html";
    wxLaunchDefaultBrowser(Page);
  #else
    Page=GetDataDir();
    if(Lan==wxLANGUAGE_ITALIAN)Page+=L"/bilancio_it.html";
    else Page+=L"/bilancio_en.html";
    wxLaunchDefaultBrowser(Page);
  #endif
}

void BilancioMainFrame::AuthorClick(wxCommandEvent& event){
	AuthorWindow();}

void BilancioMainFrame::UpdateCalendar(wxDateTime master_mindate){
	int Month, Year;

	if(Master_Attached) calmindate = master_mindate;
	else{
		Month = Data->FileData.Month - 1;
		Year = Data->FileData.Year;
		if(Month < 1){
			Month = 12;
			Year--;}
		calmindate.Set(25, wxDateTime::Month(Month - 1), Year, 0, 0, 0, 0);}

	Month = Data->FileData.Month + 1;
	Year = Data->FileData.Year;
	if(Month > 12){
		Month = 1;
		Year++;}
  calmaxdate.Set(5, wxDateTime::Month(Month - 1), Year, 23, 59, 59, 999);

  if(calmaxdate < wxDateTime::Today()) calmaxdate = wxDateTime::Today();

  CheckDate();}

#if(wxCHECK_VERSION(3,0,0))
	void BilancioMainFrame::GiveTime(wxDateEvent& event){
		Data->Hour = TimePicker->GetValue();}
#else
	void BilancioMainFrame::GiveTime(void){
		wxDateTime time;
		wxString timestring;

		timestring=TimePicker->GetValue();
		time.ParseTime(TimePicker->GetValue());
		#ifdef __WXMSW__
			if(timestring.Find("PM")!=wxNOT_FOUND){
		#else
			if(timestring.Find(L"PM")!=wxNOT_FOUND){
		#endif // __WXMSW__
			time.Add(wxTimeSpan::Hours(12));}
		Data->Hour=time;
	}
#endif

void BilancioMainFrame::UpdateMenu(void){
	bool C=!Data->FileData.FileView.IsEmpty();
	bool M=Data->FileData.Modified;
	bool W=!Data->FileData.ReadOnly;
	if(C&W){
        if(!menu_opendoc){
            // File menu update ------------------------
            MainMenu_File_->Insert(4,Revert);
            MainMenu_File_->Insert(5,XML);
            /*
            // TODO (igor#1#): To be restored when SQL encryption is available
            MainMenu_File_->Insert(6, ModifyPassword);
            MainMenu_File_->InsertSeparator(6);
            */

            // -----------------------------------------
            //MAINMENU->Insert(1,MainMenu_Edit_,_("Edit"));
            MAINMENU->Insert(2,MainMenu_Funds_,_("Funds"));
            MAINMENU->Insert(3,MainMenu_Operations_,_("Operations"));
            MAINMENU->Insert(4,MainMenu_Objects_,_("Objects"));
            MAINMENU->Insert(5,MainMenu_Credits_,_("Credits"));
            MAINMENU->Insert(6,MainMenu_Debts_,_("Debts"));
            MAINMENU->Insert(7,MainMenu_ShopList_,_("Shopping list"));
            InitLanguage();
            menu_opendoc=true;}}
        else {
            if(menu_opendoc){
                // File menu update ------------------------
                MainMenu_File_->Remove(bilRevert);
                /*
                // TODO (igor#1#): To be restored when SQL encryption is available
                MainMenu_File_->Remove(bilModifyPassword);
                MainMenu_File_->Remove(MainMenu_File_->FindItemByPosition(MainMenu_File_->GetMenuItemCount()-2)); // password_separator removal
                */

                // -----------------------------------------
                for(int i = 0; i < 6; i++) MAINMENU->Remove(2);
                	// Funds menu removal
                	// Operations menu removal
                	// Objects menu removal
                	// Credits menu removal
                	// Debts menu removal
                	// Shoplist menu removal
                menu_opendoc=false;}}
    if(C)
    	if(menu_opendoc)
        MainMenu_File_->Enable(bilRevert,M);

		#ifdef __WXMSW__
			if(MainMenu_Help_->FindItem(2000) == NULL){
				MainMenu_Help_->Insert(3, 2000, _("License"), wxEmptyString, NULL);
				Connect(2000, wxEVT_COMMAND_MENU_SELECTED,wxCommandEventHandler(BilancioMainFrame::LicenseClick));
			}
		#endif // __WXMSW__

	// Toolbar update
  ToolBar->EnableTool(bilXML,C);
  ToolBar->EnableTool(bilNewFund,C&W);
  ToolBar->EnableTool(bilRemoveFund,C&W);
  ToolBar->EnableTool(bilResetFund,C&W);
  ToolBar->EnableTool(bilGain,C&W);
  ToolBar->EnableTool(bilExpense,C&W);
  ToolBar->EnableTool(bilCategory,C&W);
  ToolBar->EnableTool(bilReceived,C&W);
  ToolBar->EnableTool(bilGifted,C&W);
  ToolBar->EnableTool(bilLend,C&W);
  ToolBar->EnableTool(bilGetBack,C&W);
  ToolBar->EnableTool(bilBorrow,C&W);
  ToolBar->EnableTool(bilGiveBack,C&W);
  ToolBar->EnableTool(bilNewCredit,C&W);
  ToolBar->EnableTool(bilRemoveCredit,C&W);
  ToolBar->EnableTool(bilCondoneCredit,C&W);
  ToolBar->EnableTool(bilNewDebt,C&W);
  ToolBar->EnableTool(bilRemoveDebt,C&W);
  ToolBar->EnableTool(bilCondoneDebt,C&W);
  ToolBar->EnableTool(bilAddShopItem,C&W);
  ToolBar->EnableTool(bilDelShopItem,C&W);

	Calendar->Enable(C&W);
	TimePicker->Enable(C&W);
}

bool BilancioMainFrame::SaveDatabase(void){
	wxString FD;
	if(Data->FileData.Modified){
		/*
		if(Data->FileData.FileName.IsEmpty())FD=GetValue();
		else */FD = Data->FileData.FileView;
		wxString Msg = wxString::Format(_("Save changes to %s?"), FD.c_str());
		wxBell();
		switch(wxMessageBox(Msg, _("Bilancio"), wxYES_NO | wxCANCEL)){
			case wxYES:
				Data->database->ReleaseSavepoint(L"rollback");
			case wxNO:
				break;
			case wxCANCEL:
				return false;
			default:
				Error(5, wxEmptyString);
				return false;}}
	return true;}

void BilancioMainFrame::FormClose(wxCloseEvent &event){
	if(Data->FileData.Modified){
		if(! SaveDatabase()){
			event.Veto(true);
			return;}
		/*
		else{

		}
		*/
	}
	else Data->database->Rollback(L"rollback");
	FormDestroy();
	Destroy();
}

void BilancioMainFrame::FormDestroy(void){
	Initialised=false;
	delete FundChart;
	delete TrendChart;
	TreeNames->Clear();
	delete TreeNames;
	delete FindDialog;}	// Necessary to grant BilApp exits

bool BilancioMainFrame::AutoOpen(void){

	bool new_document = false;
	wxString Document = GetDefaultDocument();
	if(Document.IsEmpty()){
		// Create document filename
		Document = GetBilDocPath();
		if(Document.IsEmpty()) Document = GetOSDocDir();
		Document += L"/bilancio.bil";
		new_document = true;
		// Set default document
		Set_DefaultDocument(Document);
	}
	wxLogMessage(L"Database: " + Document);

	if(wxFileExists(Document)){
		if(new_document){
			Error(36, wxEmptyString);
			Close(false);
			return false;
		}
	}
	else switch(RunWizard(Progress, Document, true)){	// create default document
		case 2:
			Set_DefaultDocument(Document);
			break;
		default:
			Close(false);
			return false;
	}

	// open the document
	if(! Data->OpenDatabase(Document)){
		wxLogMessage(L"Error opening database!");
		return false;}

	ShowControls(true);
	#ifdef _BIL_CHART_WXFREECHART
		ShowF();
	#endif // _BIL_CHART_WXFREECHART
	return true;}

void BilancioMainFrame::ShowFindDialog (wxCommandEvent& event){
	if (FindDialog == NULL)FindDialog=new wxFindReplaceDialog(this,FindData,_("Find"), wxFR_NOUPDOWN | wxFR_NOMATCHCASE | wxFR_NOWHOLEWORD);
	FindDialog->Show();}

void BilancioMainFrame::Find(wxFindDialogEvent& event){	// to be debugged
	bool Found=false;
	bool loop=false;	// set to true on search from top
	int /*i,*/r,start,stop;	// list indexes
	int findex;	// find index (>0 if string is found)
	wxString find_str=event.GetFindString().MakeLower();
	r=GetFirstSelectedRow();
	if(r<0)return;

	start=r+1;
	stop = Data->NLin;

	MainFindLoop:
	findex=FindLoop(start,stop,find_str);
	if(findex>=0)Found=true;
	else if(!loop){
		start=0;
		stop=r+1;
		loop=true;
		goto MainFindLoop;}
	if(Found){
		wxBell();
		Report->SelectRow(findex,false);
		Report->MakeCellVisible(findex,0);}
	else{
		wxString Msg=wxString::Format(_("\"%s\" not found."),find_str.c_str());
		wxBell();
		::wxMessageBox(Msg,_("Bilancio"),wxOK);}}

int BilancioMainFrame::FindLoop(int StartIndex, int StopIndex, wxString find_str){
	int f1, f2, f3;	// string find results
	long cat_index;
	wxString S1, S2, S3;
	for(int i = StartIndex; i < StopIndex; i++){
		S1 = Data->Lines[i].Value.MakeLower();
		S2 = Data->Lines[i].Reason.MakeLower();
		cat_index = Data->Lines[i].CategoryIndex;
		if(cat_index >= 0){
			for(int j = 0; j < Data->NCat; j++) if(Data->Categories[j].Id == cat_index){
				S3 = Data->Categories[j].Name.MakeLower();
				break;}}
		else S3 = wxEmptyString;
		f1 = S1.Find(find_str);
		f2 = S2.Find(find_str);
		f3 = S3.Find(find_str);
		if(! (f1 == wxNOT_FOUND && f2 == wxNOT_FOUND && f3 == wxNOT_FOUND)) return i;}
	return -1;}

void BilancioMainFrame::FindClose(wxFindDialogEvent& event){
	FindDialog->Hide();}

void BilancioMainFrame::WebSiteClick(wxCommandEvent& event){
	wxLaunchDefaultBrowser(GetWebAddress());}

void BilancioMainFrame::BugClick(wxCommandEvent& event){
	wxLaunchDefaultBrowser(GetBugAddress());}

void BilancioMainFrame::DonateClick(wxCommandEvent& event){
	DonateWindow();}

/*
void BilancioMainFrame::ResizeCols(int C){
  int Current,Max;
  if(C>Report->GetNumberCols())return;
	Max=0;
  for(int i=0;i<Report->GetNumberRows();i++){
    Current=Report->Canvas->TextWidth(Report->Cells[C][i]);
    if((C==1)||(C==3))Current+=5; // Necessary for time widths
    if(Max<Current)Max=Current;}
  if(Max!=Report->ColWidths[C])Report->ColWidths[C]=Max+5;
  Report->Repaint();}
*/

void BilancioMainFrame::CheckDate(wxCalendarEvent& event){
	CheckDate();
 	if(Master_Attached) ShowMaster();}

void BilancioMainFrame::PopRepPopup( wxGridEvent& event ){
	if(Data->NLin < 1)return;
	int Row=event.GetRow();
	Report->SelectRow(Row,false);
	if(Data->IsDate(Row))return;
	Report->PopupMenu(PopRep,event.GetPosition());}

wxString BilancioMainFrame::CreateTitle(bool type){
	// set type to true to make modification and read-only checks
	wxString label = _("Monthly report of ");
	if(Master_Attached) label += IntToMonth(Calendar->GetDate().GetMonth() + 1);
	else label += IntToMonth(Data->FileData.Month);
	label += char(32);

	if(Master_Attached) label += ::wxString::Format(L"%d", Calendar->GetDate().GetYear());
	else
		#ifdef __WXMSW__
			label += ::wxString::Format("%d", Data->FileData.Year);
		#else
			label += ::wxString::Format(L"%d", Data->FileData.Year);
		#endif // __WXMSW__
	if(type){
		if(Master_Attached){
			wxDateTime cal_date = Calendar->GetDate();
			int month = cal_date.GetMonth();
			int year = cal_date.GetYear();
			if((year < Data->FileData.Year) || ((month < Data->FileData.Month - 1) && (year == Data->FileData.Year)) || (year > Data->FileData.Year))
				label = label + _(" [Read-only]");
		}
		else if(Data->FileData.Modified) label += L" [*]";
	}
	return label;
}

void BilancioMainFrame::ExtClick(wxCommandEvent& event){
	int id = event.GetId() - bilExtTool;
	#ifdef __WXMSW__
		::wxShell(L"\"" + GetTool(id) + L"\"");
	#else
		::wxExecute(L"xdg-open \""+GetTool(id)+L"\"",wxEXEC_ASYNC,NULL);
	#endif
}

void BilancioMainFrame::onResize(wxSizeEvent& event){
	wxRect rect;
	StatusBar->GetFieldRect(1, rect);
	Progress->SetSize(rect.GetSize());
	Progress->SetPosition(rect.GetPosition());
	Progress->CenterOnParent(wxVERTICAL);
	event.Skip();}

void BilancioMainFrame::EditCategoriesClick(wxCommandEvent& event){
	int i;

	wxEditCategories *edit = new wxEditCategories(this);
	for(i = 0; i < Data->NCat; i++) edit->CategoryList->Append(Data->Categories[i].Name);

	if(edit->ShowModal() == wxID_OK){
		if(edit->modified){
			Data->UpdateCategories(edit->CategoryList);
			ShowF(false);}}
	delete edit;}

void BilancioMainFrame::remove_obtained_shopitem(wxString obj){
	wxString S1;
	for(int i = 0; i < Data->NSho; i++){
		S1 = Data->ShopItems[i].Name.MakeLower();
		if(obj == S1){
			wxString Msg = wxString::Format(_("Do you want to remove item %s from the shopping list?"), S1.c_str());
			wxBell();
			if(wxMessageBox(Msg, _("Bilancio"), wxYES_NO) == wxYES) Data->DelShopItem(i);
			break;}}}

#ifdef __WXMSW__
  void BilancioMainFrame::LicenseClick(wxCommandEvent& event){
    wxString LicenseFile = GetInstallationPath();
    switch(Lan){
      case wxLANGUAGE_ITALIAN:
        LicenseFile += "\\licenza.txt";
        break;
      default:
        LicenseFile += "\\license.txt";}
    ::wxShell(LicenseFile);}
#endif // __WXMSW__

void BilancioMainFrame::ViewArchiveClick(wxCommandEvent& event){
	if(ViewArchive->IsChecked()){

		wxString master = Get_MasterDB();
		// check if master exists
		if(! ::wxFileExists(master)){
			ViewArchive->Check(false);
			return;
		}

		// release savepoint
		if(Data->FileData.Modified){
			if(! SaveDatabase()){
				ViewArchive->Check(false);
				return;}
		}
		else Data->database->ReleaseSavepoint(L"rollback");

		// attach master archive
		#if( wxCHECK_VERSION(3,0,0) )
			Data->database->ExecuteUpdate(wxString::Format("ATTACH DATABASE '%s' AS master;", master));
		#else
			Data->database->ExecuteUpdate(wxString::Format(L"ATTACH DATABASE '%s' AS master;", master.c_str()));
		#endif
		Data->database->Savepoint(L"rollback");
		Master_Attached = true;

		SearchMasterMinDate();

	}
	else{
		// release savepoint
		if(Data->FileData.Modified){
			if(! SaveDatabase()){
				ViewArchive->Check(false);
				return;}
		}
		else Data->database->ReleaseSavepoint(L"rollback");

		// detach master database
		Data->database->ExecuteUpdate("DETACH DATABASE 'master';");
		Data->database->Savepoint(L"rollback");
		Master_Attached = false;

		// udpate calendar control minimum date
		UpdateCalendar(wxInvalidDateTime);

		ShowF();
	}
}

void BilancioMainFrame::ShowMaster(void){

	int i;

	int min_date, max_date;
	wxDateTime cal_date, min_datetime, max_datetime;
	cal_date = Calendar->GetDate();
	int month = cal_date.GetMonth();
	int year = cal_date.GetYear();

	if(((month >= Data->FileData.Month - 1) && (year == Data->FileData.Year)) || (year > Data->FileData.Year)){
		ShowF();
		return;
	}

	min_datetime = cal_date;
	min_datetime.SetDay(1);
	min_datetime.SetHour(0);
	min_datetime.SetMinute(0);
	min_datetime.SetSecond(0);
	min_datetime.SetMillisecond(0);
	min_date = min_datetime.GetTicks();

	max_datetime = cal_date;
	max_datetime.SetToLastMonthDay(max_datetime.GetMonth(), year);
	max_datetime.SetHour(23);
	max_datetime.SetMinute(59);
	max_datetime.SetSecond(59);
	max_datetime.SetMillisecond(999);
	max_date = max_datetime.GetTicks();

	wxString trail_name = L"_" + ::wxString::Format(L"%d", year) + L"_" + ::wxString::Format(L"%02d", int(month) + 1);

	Title->SetLabel(CreateTitle(true));

	double value, total;
	wxSQLite3Table fund_table, transaction_table, table;

	// TreeList construction
	ItemList->DeleteAllItems();
	ResetTree();

	bool has_funds;
	if(Data->database->TableExists(L"Funds" + trail_name, L"master")){
		has_funds = true;
		total = 0;
		#if (wxCHECK_VERSION(3, 0, 0) )
			fund_table = Data->database->GetTable(::wxString::Format("select * from master.Funds%s order by name;", trail_name));
		#else
			fund_table = Data->database->GetTable(::wxString::Format(L"select * from master.Funds%s order by name;", trail_name.c_str()));
		#endif
		for(i = 0; i < fund_table.GetRowCount(); i++){
			fund_table.SetRow(i);
			value = fund_table.GetDouble(L"value", 0);
			AddNode(Funds_tid, fund_table.GetString(L"name", wxEmptyString), FormDigits(value));
			total += value;
		}
		AddNode(Funds_tid, TreeNames->Item(6), FormDigits(total));
	}
	else has_funds = false;

	if(Data->database->TableExists(L"Credits" + trail_name, L"master")){
		total = 0;
		#if (wxCHECK_VERSION(3, 0, 0) )
			table = Data->database->GetTable(::wxString::Format("select * from master.Credits%s order by name;", trail_name));
		#else
			table = Data->database->GetTable(::wxString::Format(L"select * from master.Credits%s order by name;", trail_name.c_str()));
		#endif
		for(i = 0; i < table.GetRowCount(); i++){
			table.SetRow(i);
			value = table.GetDouble(L"value", 0);
			AddNode(Creds_tid, table.GetString(L"name", wxEmptyString), FormDigits(value));
			total += value;
		}
		AddNode(Creds_tid, TreeNames->Item(6), FormDigits(total));
	}
	if(Data->database->TableExists(L"Debts" + trail_name, L"master")){
		total = 0;
		#if (wxCHECK_VERSION(3, 0, 0) )
			table = Data->database->GetTable(::wxString::Format("select * from master.Debts%s order by name;", trail_name));
		#else
			table = Data->database->GetTable(::wxString::Format(L"select * from master.Debts%s order by name;", trail_name.c_str()));
		#endif
		for(i = 0; i < table.GetRowCount(); i++){
			table.SetRow(i);
			value = table.GetDouble(L"value", 0);
			AddNode(Debts_tid, table.GetString(L"name", wxEmptyString), FormDigits(value));
		}
		AddNode(Debts_tid, TreeNames->Item(6), FormDigits(total));
	}
	if(Data->database->TableExists(L"Loans" + trail_name, L"master")){
		#if (wxCHECK_VERSION(3, 0, 0) )
			table = Data->database->GetTable(::wxString::Format("select * from master.Loans%s order by name;", trail_name));
		#else
			table = Data->database->GetTable(::wxString::Format(L"select * from master.Loans%s order by name;", trail_name.c_str()));
		#endif
		for(i = 0; i < table.GetRowCount(); i++){
			table.SetRow(i);
			AddNode(Lends_tid, table.GetString(L"name", wxEmptyString), table.GetString(L"item", wxEmptyString));
		}
	}
	if(Data->database->TableExists(L"Borrows" + trail_name, L"master")){
		#if (wxCHECK_VERSION(3, 0, 0) )
			table = Data->database->GetTable(::wxString::Format("select * from master.Borrows%s order by name;", trail_name));
		#else
			table = Data->database->GetTable(::wxString::Format(L"select * from master.Borrows%s order by name;", trail_name.c_str()));
		#endif
		for(i = 0; i < table.GetRowCount(); i++){
			table.SetRow(i);
			AddNode(Borrowed_tid, table.GetString(L"name", wxEmptyString), table.GetString(L"item", wxEmptyString));
		}
	}

	// show report
	bool is_date;
	int cat_index;
	wxString value_string;
	wxDateTime date, time;

	if(Report->GetNumberRows()>0)Report->DeleteRows(0,Report->GetNumberRows(),false);
	Report->SetDefaultCellTextColour(wxColour(0x00, 0xAF, 0x00));

	transaction_table = Data->database->GetTable(::wxString::Format(L"SELECT * FROM master.Transactions WHERE date >= %d and date <= %d order by date;", min_date, max_date));

	Report->AppendRows(transaction_table.GetRowCount(), false);

	wxFont font = Report->GetCellFont(0, 0);
	#if (wxCHECK_VERSION(3, 0, 0) )
		font.MakeBold();
	#endif

	for(i = 0; i < transaction_table.GetRowCount(); i++){
		transaction_table.SetRow(i);
		is_date = transaction_table.GetInt(L"isdate", false);
		if(is_date){
			date = wxDateTime((time_t) transaction_table.GetInt(2, 0));
			Report->SetCellValue(i, 0, date.FormatDate());
			Report->SetCellFont(i, 0, font);
			Report->SetCellValue(i, 2, GetLogString(0));
			Report->SetCellFont(i, 2, font);
			Report->SetCellFont(i, 3, font);}
		else{
			time = wxDateTime((time_t) transaction_table.GetInt(3, 0));
			Report->SetCellValue(i, 1, time.Format(L"%H:%M", wxDateTime::Local));
			Report->SetCellValue(i, 2, GetLogString(transaction_table.GetInt(4, 0)));
			Report->SetCellValue(i, 4, transaction_table.GetString(6, wxEmptyString));
			cat_index = transaction_table.GetInt(7, 0);
			if(cat_index != -1)
				for(int j = 0; j < Data->NCat; j++) if(Data->Categories[j].Id == cat_index){
					Report->SetCellValue(i, 5, Data->Categories[j].Name);
					break;}
				}
		value_string = transaction_table.GetString(5, wxEmptyString);
		if(transaction_table.GetInt(4, 0) < 9){
			double cur;

			#if( wxCHECK_VERSION(3,0,0) )
				value_string.ToCDouble(&cur);
			#else
				value_string.ToDouble(&cur);
			#endif

			Report->SetCellValue(i, 3, FormDigits(cur));
			if(Report->GetCellValue(i, 3).IsEmpty())
				Report->SetCellValue(i, 3, _("Zero"));}
		else Report->SetCellValue(i, 3, value_string);
		Report->SetCellAlignment(i,3,wxALIGN_RIGHT,wxALIGN_CENTRE);}

	Report->AutoSizeColumns(false);
	Report->SelectRow(Report->GetNumberRows() - 1, false);
	Report->MakeCellVisible(Report->GetNumberRows() - 1, 0);

	TrendChart->Show(ShowTrendChart());
	FundChart->Show(ShowFundChart());
	if(TrendChart->IsShown()||FundChart->IsShown())PaintMasterChart(has_funds, fund_table, transaction_table);

	Update();}

void BilancioMainFrame::PaintMasterChart(bool hasfunds, wxSQLite3Table funds, wxSQLite3Table transactions){
	int i;
	if(ShowFundChart() && hasfunds){
		int fund_number = funds.GetRowCount();

		#ifdef _BIL_CHART_MATLIBPLOT
			double sum_small_funds = 0, tot_funds = 0;
			//bool default_is_small = false;

			wxString tmp_file, data_fname, chart_fname, line;
			wxString Proc = L"python " + GetDataDir();
			Proc += L"pie.py ";
			Proc += L"/usr/share/locale ";	// TODO (igor#1#): Check if it is possible to get this path programmatically

			tmp_file = GetTempFile();
			data_fname = ::wxFileName::CreateTempFileName(tmp_file);
			wxTextFile *file = new wxTextFile(data_fname);
			file->Create();

			for(i = 0; i < fund_number; i++){
				funds.SetRow(i);
				tot_funds += funds.GetDouble(L"value", 0);}

			for(i = 0; i < fund_number; i++){
				funds.SetRow(i);
				if((funds.GetDouble(L"value", 0) / tot_funds) > 0.01){
					#if(wxCHECK_VERSION(3,0,0))
						line = funds.GetString("name", wxEmptyString) + ", " + wxString::FromCDouble(funds.GetDouble("value", 0), 2) + ", 0";
					#else
						line = funds.GetString(L"name", wxEmptyString) + L", " + FormDigits(funds.GetDouble(L"value", 0)) + L", 0";
					#endif
					file->AddLine(line);}
				else sum_small_funds += funds.GetDouble(L"value", 0);
			}
			if(sum_small_funds){
				#if(wxCHECK_VERSION(3,0,0))
					line = _("Others") + ", " + wxString::FromCDouble(sum_small_funds, 2) + ", 0";
				#else
					line = _("Others");
					line += L", ";
					line += FormDigits(sum_small_funds);
					line += L", 0";
				#endif
				file->AddLine(line);
			}

			file->Write(wxTextFileType_Unix,wxConvUTF8);
			// file closure
			file->Close();
			delete file;

			Proc += data_fname + L" ";

			chart_fname = ::wxFileName::CreateTempFileName(tmp_file);
			::wxRemoveFile(chart_fname);
			chart_fname += L".png";
			Proc += chart_fname;

			wxExecute(Proc, wxEXEC_SYNC);
			::wxRemoveFile(data_fname);

			FundChart->SetBitmap(GetScaledBitmap(chart_fname, FundChart));

			::wxRemoveFile(chart_fname);

		#elif defined ( _BIL_CHART_WXFREECHART )
			// Fund Chart construction


			int cindex=0;	// colour array index
			PiePlot *FundPlot=new PiePlot();
			FundPlot->Set3DView(true);
			FundPlot->SetEllipticAspect(.6);
			double fundvalues[fund_number];
			wxString fundnames[fund_number];
			wxColour colours[fund_number];

			for(i = 0; i < fund_number; i++){
				funds.SetRow(i);
				fundvalues[i] = funds.GetDouble("value", 0);
				fundnames[i] = funds.GetString("name", wxEmptyString);}
			CategorySimpleDataset* dataset = new CategorySimpleDataset(fundnames, WXSIZEOF(fundnames));
			dataset->AddSerie(_("Funds"),fundvalues, WXSIZEOF(fundvalues));
			// colour array initialization
			i=0;
			while(cindex < fund_number){
				switch(i){
					case 1:
					colours[cindex]=wxColour(0xFF,0xFF, 0x99);
					break;
					case 2:
					colours[cindex]=wxColour(0x3D, 0xEB, 0x3D);
					break;
					case 3:
					colours[cindex]=wxColour(0xFF, 0x00, 0x00);
					break;
					case 4:
					colours[cindex]=wxColour(0x00, 0xFF, 0x00);
					break;
					case 5:
					colours[cindex]=wxColour(0x00, 0x00, 0xFF);
					break;
					default:
					colours[cindex]=wxColour(0x99, 0xCC, 0xFF);
					i=0;}
				i++;
				cindex++;}
			ColorScheme* colorScheme = new ColorScheme(colours, WXSIZEOF(colours)); FundPlot->SetDataset(dataset);
			FundPlot->SetColorScheme(colorScheme);
			FundChart->SetChart(new Chart(FundPlot, _("Fund Chart")));
		#endif // _BIL_CHART_MATLIBPLOT
	}

	if(ShowTrendChart()){
		// Trend chart construction
		wxDateTime LastDate = Calendar->GetDate();
		LastDate.SetDay(1);
		int number_of_days = wxDateTime::GetNumberOfDays(LastDate.GetMonth(), LastDate.GetYear(), wxDateTime::Gregorian);

		double trendvalues[number_of_days];
		for(i = 0; i < number_of_days; i++) trendvalues[i] = -1;
		const wxChar *trenddates[number_of_days];
		time_t times[WXSIZEOF(trenddates)];
		double	LastValue = 0,
						MinValue = 0,
						MaxValue = 0;

		wxDateTime date, dt;

		int j = 0;
		for(i = 0; i < transactions.GetRowCount(); i++){
			transactions.SetRow(i);
			if(transactions.GetInt(L"isdate", false)){
				date = wxDateTime((time_t) transactions.GetInt(2, 0));
				j = date.GetDay() - 1;
				LastDate = date;
				#if (wxCHECK_VERSION(3, 0, 0) )
					transactions.GetString(5, wxEmptyString).ToCDouble(&LastValue);
				#else
					transactions.GetString(5, wxEmptyString).ToDouble(&LastValue);
				#endif
				trendvalues[j] = LastValue;
				times[j] = LastDate.GetTicks();

				if(LastValue < MinValue) MinValue = LastValue;
				if(LastValue > MaxValue) MaxValue = LastValue;
			}}

		bool	found_before = false,
					found_after;
		int k;
		for(i = 0; i < number_of_days; i++){
			if(trendvalues[i] == -1){
				if(found_before){
					found_after = false;
					for(k = i+1; k < number_of_days; k++)
						if(trendvalues[k] != -1){
							found_after = true;
							break;}

					if(found_after) trendvalues[i] = trendvalues[i-1];
					else trendvalues[i] = 0;}
				else trendvalues[i] = 0;
			dt = wxDateTime(i + 1, Calendar->GetDate().GetMonth(), Calendar->GetDate().GetYear(), 0, 0, 0, 0);
			times[i] = dt.GetTicks();}
			else found_before = true;}

		#ifdef _BIL_CHART_MATLIBPLOT
			wxString tmp_file, data_fname, chart_fname, line;
			wxString Proc = L"python " + GetDataDir();
			Proc += L"trend.py ";
			Proc += L"/usr/share/locale ";	// TODO (igor#1#): Check if it is possible to get this path programmatically

			tmp_file = GetTempFile();
			data_fname = ::wxFileName::CreateTempFileName(tmp_file);
			wxTextFile *file = new wxTextFile(data_fname);
			file->Create();

			for(i = 0; i < number_of_days; i++)
				#if( wxCHECK_VERSION(3,0,0) )
					file->AddLine(wxString::Format(L"%d", int(times[i])) + L", " + wxString::FromCDouble(trendvalues[i], 2));
				#else
					file->AddLine(wxString::Format(L"%d", int(times[i])) + L", " + FormDigits(trendvalues[i]));
				#endif

			file->Write(wxTextFileType_Unix,wxConvUTF8);
			// file closure
			file->Close();
			delete file;

			Proc += data_fname + L" ";

			chart_fname = ::wxFileName::CreateTempFileName(tmp_file);
			::wxRemoveFile(chart_fname);
			chart_fname += L".png";
			Proc += chart_fname;

			wxExecute(Proc, wxEXEC_SYNC);
			::wxRemoveFile(data_fname);

			TrendChart->SetBitmap(GetScaledBitmap(chart_fname, TrendChart, true));

			::wxRemoveFile(chart_fname);

		#elif defined ( _BIL_CHART_WXFREECHART )
			XYPlot *trendplot = new XYPlot();
			wxColour color_trend = wxColour(0, 0, 0);
			XYLineRenderer *renderer_trend=new XYLineRenderer();
			renderer_trend->SetSerieColour(0,&color_trend);
			TimeSeriesDataset *dataset1 = new TimeSeriesDataset(trendvalues, times, WXSIZEOF(trendvalues));
			dataset1->SetRenderer(renderer_trend);
			trendplot->AddDataset(dataset1);

			NumberAxis *leftAxis = new NumberAxis(AXIS_LEFT);
			DateAxis *bottomAxis = new DateAxis(AXIS_BOTTOM);
			bottomAxis->SetVerticalLabelText(true);
			bottomAxis->SetDateFormat(L"%d/%m");

			// Set y scale
			if(MinValue == 0){
				for(i = 0; i < j; i++)if(trendvalues[i] > 0){
					MinValue = trendvalues[i];
					break;}}
			if(MaxValue == 0) MaxValue = trendvalues[0];
			for(i=1;i<j;i++){
				if((trendvalues[i] != -1) && (trendvalues[i] < MinValue)) MinValue=trendvalues[i];
				if(trendvalues[i]>MaxValue)MaxValue=trendvalues[i];}
			int p=0,num;
			double res=MinValue;
			while(res>=1){
				res=res/10;
				if(res>=1)p++;}
			p=pow(10,p);
			num=MinValue/p;
			if(num > 1) num--;
			MinValue=num*p;
			p=0;
			double real_max_value = res = MaxValue;
			while(res >= 1){
				res = res / 10;
				if(res >= 1) p++;}
			p = pow(10, p);
			num = MaxValue / p + 1;
			MaxValue = num * p;

			if((p > 1000) && (MaxValue / MinValue > 2)) {
				double test_max_value = MaxValue - (MinValue / 4 * 3);
				if(test_max_value >= real_max_value) MaxValue = test_max_value;
				else {
					test_max_value = MaxValue - (MinValue / 2);
					if(test_max_value >= real_max_value) MaxValue = test_max_value;}
			}

			leftAxis->SetFixedBounds(MinValue, MaxValue);

			trendplot->AddAxis(leftAxis);
			trendplot->AddAxis(bottomAxis);
			trendplot->LinkDataVerticalAxis(0, 0);
			trendplot->LinkDataHorizontalAxis(0, 0);

			TrendChart->SetChart(new Chart(trendplot, _("Trend Chart")));
		#endif // _BIL_CHART_MATLIBPLOT
	}
	FreshChartShown = false;
}

void BilancioMainFrame::ImportDocument(wxCommandEvent& event){
	wxFileDialog *Dialog=new wxFileDialog(wxTheApp->GetTopWindow(),wxEmptyString,wxEmptyString,wxEmptyString,wxEmptyString,wxFD_OPEN,wxDefaultPosition,wxDefaultSize,L"Dialog");
	Dialog->SetDirectory(GetOSDocDir());
	wxString filter = _("Bilancio documents");
	filter += L"|*.bil;*.BIL";
	Dialog->SetWildcard(filter);
	if(Dialog->ShowModal() != wxID_OK){
		delete Dialog;
		return;}

	wxString File = Dialog->GetPath();

	if(! CheckAndPromptForConversion(File)){
		delete Dialog;
		return;
	}

	wxString master = Get_MasterDB();
	wxString trailname = Data->GenerateTrailName(File);

	// release savepoint
	if(Data->FileData.Modified){
		if(! SaveDatabase()) return;
	}
	else Data->database->ReleaseSavepoint(L"rollback");

	if(Master_Attached) Data->database->ExecuteUpdate(L"DETACH DATABASE 'master';");

	/*bool has_cat = */Data->Prepare_MasterDB(master, trailname);

	#if( wxCHECK_VERSION(3,0,0) )
		Data->database->ExecuteUpdate(wxString::Format("ATTACH DATABASE '%s' AS import;", File));
	#else
		Data->database->ExecuteUpdate(wxString::Format(L"ATTACH DATABASE '%s' AS import;", File.c_str()));
	#endif
	Data->Import_inMaster(master, trailname);
	Data->database->ExecuteUpdate(L"DETACH DATABASE 'import';");

	#if( wxCHECK_VERSION(3,0,0) )
		if(Master_Attached)	Data->database->ExecuteUpdate(wxString::Format("ATTACH DATABASE '%s' AS master;", master));
	#else
		if(Master_Attached)	Data->database->ExecuteUpdate(wxString::Format(L"ATTACH DATABASE '%s' AS master;", master.c_str()));
	#endif

	Data->database->Savepoint(L"rollback");

	if(Master_Attached){
		SearchMasterMinDate();
		ShowMaster();
	}

	delete Dialog;
}

void BilancioMainFrame::SearchMasterMinDate(void){
	// look for oldest transaction
	wxSQLite3Table Table;
	Table = Data->database->GetTable("select min (date) from master.transactions;");
	Table.SetRow(0);
	int minimum = Table.GetInt(0, 0);

	// udpate calendar control minimum date
	wxDateTime min_date = wxDateTime((time_t) minimum);
	min_date.SetDay(1);
	min_date.SetHour(0);
	min_date.SetMinute(0);
	min_date.SetSecond(0);
	min_date.SetMillisecond(0);
	UpdateCalendar(min_date);
}

#ifdef _BIL_CHART_MATLIBPLOT
	wxBitmap BilancioMainFrame::GetScaledBitmap(wxString chart, wxStaticBitmap* panel, bool trend){
		int new_x, new_y;
		double scale_ratio;
		wxBitmap chart_bmp = wxBitmap(chart);
		if(trend) Trend_bmp = chart_bmp;
		else Fund_bmp = chart_bmp;
		wxImage image = chart_bmp.ConvertToImage();
		wxSize sz_client = panel->GetSize();
		wxSize sz_image;
		#if( wxCHECK_VERSION(3,0,0) )
		 	sz_image = image.GetSize();
		#else
			sz_image.Set(image.GetWidth(), image.GetHeight());
		#endif
		if(trend || (sz_client.GetX() < sz_client.GetY())){
			new_x = sz_client.GetX();
			scale_ratio = (double) sz_image.GetX() / new_x;
			if(scale_ratio || trend) new_y = sz_image.GetY() / scale_ratio;
			else{
				scale_ratio = (double) new_x / sz_image.GetX();
				new_y = sz_image.GetY() * scale_ratio;
			}
		}
		else{
			new_y = sz_client.GetY();
			scale_ratio = (double) sz_image.GetY() / new_y;
			if(scale_ratio) new_x = sz_image.GetX() / scale_ratio;
			else{
				scale_ratio = (double) new_y / sz_image.GetY();
				new_x = sz_image.GetX() * scale_ratio;
			}
		}

		if(trend) image = image.Scale(sz_client.GetX(), sz_client.GetY(), wxIMAGE_QUALITY_NORMAL);
		else image = image.Scale(new_x, new_y, wxIMAGE_QUALITY_NORMAL);
		/*funds =*/return wxBitmap(image);

	}
#endif // _BIL_CHART_MATLIBPLOT

#ifdef _BIL_CHART_MATLIBPLOT
	void BilancioMainFrame::FundResize( wxSplitterEvent& event ){
		if(FreshChartShown) PaintChart();
	}
#endif // _BIL_CHART_MATLIBPLOT

void BilancioMainFrame::CheckDate(void){
	wxDateTime date = Calendar->GetDate();
  if(date < calmindate) date = calmindate;
  else if(date > calmaxdate) date = calmaxdate;
  Calendar->SetDate(date);
 	Data->Day = date;}

wxString BilancioMainFrame::GetTempFile(void){
	wxString tempfile;
	wxStandardPathsBase& Paths=::wxStandardPaths::Get();
	#ifdef __WXMSW__
		tempfile = Paths.GetTempDir() + L"\\bil";
	#else
		tempfile = Paths.GetTempDir() + L"/bil";
	#endif // __WXMSW__
 	return tempfile;}

#endif //MAINFRAME_CPP_INCLUDED

