/***************************************************************
 * Name:      main_wx.h
 * Purpose:   Code for Bilancio Application Class
 * Author:    Igor Calì (igor.cali0@gmail.com)
 * Created:   2017-01-08
 * Copyright: Igor Calì (igor.cali0@gmail.com)
 * License:		GNU
 **************************************************************/

#ifndef WX_PRECOMP
	#include <wx/wx.h>
#else
	#include <wx/wxprec.h>
#endif

#include <wx/datetime.h>
//#include <wx/fileconf.h> // For wxFileConfig
#include <wx/calctrl.h>
#include <wx/sizer.h>
#include <wx/toolbar.h>
#include <wx/tbarbase.h>
#include <wx/gauge.h>
#include <wx/stattext.h>
#include <wx/treectrl.h>
#ifdef _BIL_CHART_WXFREECHART
	#include <wx/chartpanel.h>
#endif // _BIL_CHART_WXFREECHART
#include <wx/listbox.h>
#include <wx/grid.h>
#include <wx/statusbr.h>
#include <wx/statbmp.h>
#include <wx/fdrepdlg.h>  // For wxFindReplaceDialog

#if(wxCHECK_VERSION(3,0,0))
	#include <wx/timectrl.h>
#else
	#include "../../3rdparty/mytimectrl.h"
#endif
#if (_BIL_ICONSIZE_ == 24)
	#ifdef _BIL_CHART_MATLIBPLOT
		#include "wxmainframe_24_mpl.h"
	#else
		#include "wxmainframe_24.h"
	#endif // _BIL_CHART_MATLIBPLOT
#else
	#include "wxmainframe_16.h"
#endif // _BIL_ICONSIZE_

#include "../../bil31core.h"

#ifdef __BORLANDC__
//	#pragma hdrstop
#endif

//class MyCanvas;

const int bilExtTool=2000;

class BilancioMainFrame : public wxMainFrame{
friend class BilApp;	// for ShowF()

private:
	// GUI Controls

	#ifdef _BIL_CHART_MATLIBPLOT
		wxBitmap Fund_bmp;
		wxBitmap Trend_bmp;
	#elif defined ( _BIL_CHART_WXFREECHART )
		wxChartPanel *FundChart;
		wxChartPanel *TrendChart;
	#endif // _BIL_CHART_MATLIBPLOT

	wxFindReplaceDialog *FindDialog;

	// Non-GUI Controls
	bool	Initialised,
				menu_opendoc,  //  menu_opendoc: true if doc menus are active
				Master_Attached,
				FreshChartShown	// true if charts are drawn out of last month data
														;
	wxArrayString *TreeNames;  // TreeList branch names
	wxTreeItemId Funds_tid,Creds_tid,Debts_tid,Lends_tid,Borrowed_tid;
 	wxDateTime calmindate, calmaxdate;
	wxFindReplaceData *FindData;

	wxColour backup_report_colour;

	// Routines
	void InitLanguage(void);
	void ResetTree(void);
	void ShowControls(bool V);
	void FundOperation(int Op);
	void Operation(int Op);
	wxDateTime bilGetTime(void);
	void ObjectOperation(int Op);
	void ShopOperation(int O);
	void ObjectOperation2(int Op);
	void RebuildTools(void);
	void ShowF(bool doChart = true);
	void ShowMaster(void);
	void PaintMasterChart(bool hasfunds, wxSQLite3Table funds, wxSQLite3Table transactions);
	void SearchMasterMinDate(void);
	//void AddNode(wxString Node,wxString OptNode,wxString Name,wxString Value);
	void AddNode(wxTreeItemId Node,/*wxString OptNode,*/wxString Name,wxString Value);
	void PaintChart(void);
	void UpdateMenu(void);
	bool SaveDatabase(void);
	void FormDestroy(void);
	int GetFirstSelectedRow(void);
	//void ResizeCols(int C);
  void ReportSelect(wxGridEvent& event);
  void ReportRangeSelect(wxGridRangeSelectEvent& event);
	wxString CreateTitle(bool type);
	void remove_obtained_shopitem(wxString obj);
	#ifdef _BIL_CHART_MATLIBPLOT
		wxBitmap GetScaledBitmap(wxString chart, wxStaticBitmap* panel, bool trend = false);
	#endif // _BIL_CHART_MATLIBPLOT
	void CheckDate(void);
	wxString GetTempFile(void);
protected:
	void SaveClick(wxCommandEvent& event);
	void ViewArchiveClick(wxCommandEvent& event);
	//void ModifyPasswordClick(wxCommandEvent& event);	// TODO (igor#1#): To be restored when SQL encryption is available
	void RevertClick(wxCommandEvent& event);
	void ExitClick(wxCommandEvent& event);
	void CopyClick(wxCommandEvent& event);
	//void Find1Click(wxCommandEvent& event);
	void NewFundClick(wxCommandEvent& event);
	void RemoveFundClick(wxCommandEvent& event);
	void ResetFundClick(wxCommandEvent& event);
	void SetTotalClick(/*bool MainFrame*/wxCommandEvent& event);
	void DefaultFundClick(wxCommandEvent& event);
	void GainClick(wxCommandEvent& event);
	void ExpenseClick(wxCommandEvent& event);
	void CategoryClick(wxCommandEvent& event);
	void NewCreditClick(wxCommandEvent& event);
	void RemoveCreditClick(wxCommandEvent& event);
	void CondoneCreditClick(wxCommandEvent& event);
	void NewDebtClick(wxCommandEvent& event);
	void RemoveDebtClick(wxCommandEvent& event);
	void CondoneDebtClick(wxCommandEvent& event);
	//void ContributionsClick(wxCommandEvent& event);
	void ReceivedClick(wxCommandEvent& event);
	void GiftedClick(wxCommandEvent& event);
	void LendClick(wxCommandEvent& event);
	void BorrowClick(wxCommandEvent& event);
	void AddShopItemClick(wxCommandEvent& event);
	void DelShopItemClick(wxCommandEvent& event);
	void GetBackClick(wxCommandEvent& event);
	void GiveBackClick(wxCommandEvent& event);
	void ShowFindDialog (wxCommandEvent& event);
	void Find(wxFindDialogEvent& event);
	int FindLoop(int StartIndex,int StopIndex, wxString find_str);
	void FindClose(wxFindDialogEvent& event);
	void WebSiteClick(wxCommandEvent& event);
	void BugClick(wxCommandEvent& event);
	void DonateClick(wxCommandEvent& event);
	void CheckDate(wxCalendarEvent& event);
	void WizClick(wxCommandEvent& event);
	void OldConv1Click(wxCommandEvent& event);
	void OptionsClick(wxCommandEvent& event);
	void HelpMenuClick(wxCommandEvent& event);
	void AuthorClick(wxCommandEvent& event);
	void PopRepPopup( wxGridEvent& event );
	void ExtClick(wxCommandEvent& event);
	void onResize(wxSizeEvent& event);
	void EditCategoriesClick(wxCommandEvent& event);
	#ifdef __WXMSW__
    void LicenseClick(wxCommandEvent& event);
  #endif // __WXMSW__
 	void ImportDocument(wxCommandEvent& event);
	#ifdef _BIL_CHART_MATLIBPLOT
		void FundResize( wxSplitterEvent& event );
	#endif // _BIL_CHART_MATLIBPLOT

	#if(wxCHECK_VERSION(3,0,0))
		void GiveTime(wxDateEvent& event);
	#else
		void GiveTime(void);
	#endif

	public:
	// GUI components
	//MyCanvas *canvas;
	#if(wxCHECK_VERSION(3,0,0))
		wxTimePickerCtrl *TimePicker;
	#else
		MyTimeCtrl *TimePicker;
	#endif

	// Routines
	void Draw(wxDC& dc);
	//BilancioMainFrame(const wxString& title);
	BilancioMainFrame(wxWindow* parent);
	~BilancioMainFrame(void);
	bool AutoOpen(void);
	void UpdateCalendar(wxDateTime master_mindate);
	void TextConvClick(wxCommandEvent& event);
	void FormClose(wxCloseEvent &event);

	//DECLARE_DYNAMIC_CLASS(BilancioMainFrame)
	//DECLARE_EVENT_TABLE()
};

//extern wxString Website;

/*
#ifdef _BIL_INSTALLEDUPDATE
	WXIMPORT bool GetCheckUpdates(void);
#endif
*/
WXIMPORT bool ShowBar(void);
WXIMPORT void Set_DefaultDocument(wxString Document);
WXIMPORT void DoFirstRun(void);
WXIMPORT void SetLastFile(wxString FName);
WXIMPORT wxString GetLastFile(void);
WXIMPORT bool ShowOptionsDialog(void);
WXIMPORT bool ShowTrendChart(void);
WXIMPORT bool ShowFundChart(void);
WXIMPORT bool GetAutoOpen(void);
WXIMPORT bool IsFirstRunEver(void);
WXIMPORT wxString GetDefaultDocument(void);
WXIMPORT int GetToolCount(void);
WXIMPORT wxString GetTool(int i);
WXIMPORT void DoFirstRun(void);
WXIMPORT int RunWizard(wxGauge* Prog, wxString &S, bool def);
WXIMPORT void Logo(wxString Product);
//WXIMPORT void About(wxString Name,wxString Ver,wxString Date);
WXIMPORT void AuthorWindow(void);
WXIMPORT void DonateWindow(void);
WXIMPORT int GetLang(void);
WXIMPORT bool CheckAndPromptForConversion(wxString File);

#ifdef __WXMSW__
  // Calls to module bilopt.dll
  WXIMPORT wxString GetInstallationPath(void);
  WXIMPORT wxString GetDataDir(void);
#endif

WXIMPORT /*const*/ wxString GetWebAddress(void);
WXIMPORT /*const*/ wxString GetBugAddress(void);
