/***************************************************************
 * Name:      bilconvert.cpp
 * Purpose:   old format conversion tool for Bilancio
 * Author:    Igor Calì (igor.cali0@gmail.com)
 * Created:   2016-10-09
 * Copyright: Igor Calì (http://igisw-bilancio.sourceforge.net/)
 * License:		GNU
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include "../constants.h"
#include "bilconvert.h"
#include "dataconverter.h"

wxLocale bil_locale;

#ifdef __WXMSW__
  wxIMPLEMENT_APP(bilconvert);
#else
  #if(wxCHECK_VERSION(3,0,0))
  	wxIMPLEMENT_APP_CONSOLE(bilconvert);
	#else
		IMPLEMENT_APP(bilconvert);
	#endif
#endif // __WXMSW__

bool bilconvert::OnInit(){

	#ifdef __WXMSW__
    wxLocale::AddCatalogLookupPathPrefix(GetShareDir());
	#else
    wxLocale::AddCatalogLookupPathPrefix(GetShareDir()+L"locale"/*/it/LC_MESSAGES"*/);
    //wxLocale::AddCatalogLookupPathPrefix(L"/usr/share/locale"/*/it/LC_MESSAGES"*/);
    		// NOTE (igor#1#): 2014/08/16 - Monitor the path at line above; it worked well previously with full path (now commented).
    		//                 Is it related to wxWidgets 3.0.1 installed in Ubuntu from tarball since debian packages not available?
	#endif // __WXMSW__
	bil_locale.Init(wxLANGUAGE_DEFAULT,wxLOCALE_LOAD_DEFAULT);
	bil_locale.AddCatalog(L"bilancio");

	// TODO (igor#1#): ADD COMMAND LINE PARSER

  #ifdef __WXMSW__
    // Create top level window (hidden)
    wxFrame *frame = new wxFrame (NULL, 100, "bilconvert", wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE,
                         "wxFrameNameStr");
    wxTheApp->SetTopWindow(frame);
  #endif // __WXMSW__

	int arg_num = wxTheApp->argc;
	#if(wxCHECK_VERSION(3,0,0))
		const char *output_string;
	#endif

  #ifdef __WXMSW__
    printf("\n");
  #endif // __WXMSW__

	if(arg_num < 2){
		#if(wxCHECK_VERSION(3,0,0))
			output_string = _("Syntax: bilconvert <filename>\n").c_str();
			printf("%s", output_string);
		#endif
		return false;
	}

	int original_format;
	int nfunds = 0, ncredits = 0, ndebits = 0, nloans = 0, nborrows = 0, ntransactions = 0, ncategories = 0, nshoplists = 0;
	wxString default_fund = L"default";

	int i;

 	for(i = 1; i < arg_num; i++){
		wxString file = wxTheApp->argv[i];
 		if(::wxFileExists(file)){
 			dataconverter *original = new dataconverter();
			wxString result;
			original_format = CheckFileFormat(file);
			switch(original_format){
				case bilFFORMAT_301:
					original->CategoryDB = new wxArrayString();
					original->CategoryDB->Clear();

					result = original->LoadFromFile_0301(file, true);

					nfunds = original->NFun;
					ncredits = original->NCre;
					ndebits = original->NDeb;
					nloans = original->NLen;
					nborrows = original->NBor;
					ntransactions = original->NLin;
					//ncategories = original->NCat;
					nshoplists = original->NSho;
					default_fund = original->FileData.DefFund;
					break;
				case bilFFORMAT_302:
					// trim empty document
					original->CategoryDB = new wxArrayString();
					original->CategoryDB->Clear();
					original->NCat = 0;

					result = original->LoadFromFile_0302(file, true);

					nfunds = original->NFun;
					ncredits = original->NCre;
					ndebits = original->NDeb;
					nloans = original->NLen;
					nborrows = original->NBor;
					ntransactions = original->NLin;
					ncategories = original->NCat;
					nshoplists = original->NSho;
					default_fund = original->FileData.DefFund;
					break;
				default:
					result = wxEmptyString;
			}
			if(result.IsEmpty()){
 			  #ifdef __WXMSW__
          wxMessageBox(_("Error loading the file.\n"), "bilconvert", wxICON_ERROR);
        #else
          #if(wxCHECK_VERSION(3,0,0))
            output_string = _("Error loading the file.\n").c_str();
            printf("%s", output_string);
          #endif
 			  #endif // __WXMSW__
 				return false;}
			wxString vol, FilePath, FileName, ext, newfile;
			::wxFileName::SplitPath(file, &vol, &FilePath, &FileName, &ext, wxPATH_NATIVE);
			#if(wxCHECK_VERSION(3,0,0))
				ext = ".bak";
			#else
				ext = L".bak";
			#endif
			#ifdef __WXMSW__
        newfile = vol + L":" + FilePath + L"\\" + FileName + ext;
      #else
        newfile = vol + FilePath + L"/" + FileName + ext;
      #endif // __WXMSW__
 			::wxRenameFile(file, newfile, false);

			switch(original_format){
				case bilFFORMAT_301:
					// Document conversion from format (3) to (4)
					// Build CategoryDB from Remarks
					int index;
					wxString rmrk;
					original->CategoryDB->Clear();
					for(i = 0; i < original->NLin; i ++){
						if(! original->IsDate(i)){
							index = -1;
							if(! original->Lines[i].Remark0301.IsEmpty()){
								rmrk = original->Lines[i].Remark0301.MakeLower();
								for(unsigned int j = 0; j < original->CategoryDB->Count(); j ++){
									if(rmrk == original->CategoryDB->Item(j)){
										index = j;
										break;
									}
								}
								if(index == -1) index = original->CategoryDB->Add(rmrk, 1);
							}
							original->Lines[i].CategoryIndex = index;
						}
					}
					ncategories = original->CategoryDB->Count();

					original->FileData.Modified = true;
					break;}

			// Build database
			original->OpenDatabase(file);
			switch(original_format){
				case bilFFORMAT_301:
					original->LoadFromFile_0301(file, true);
					break;
				case bilFFORMAT_302:
					original->LoadFromFile_0302(file, true);
			}

			original->BuildDatabase(nfunds, ncredits, ndebits, nloans, nborrows, ntransactions, ncategories, nshoplists, default_fund);

			#ifdef __WXMSW__
				wxMessageBox(_("File successfully converted.\n"), "bilconvert", wxICON_INFORMATION);
			#else
				#if(wxCHECK_VERSION(3,0,0))
					output_string = _("File successfully converted.\n").c_str();
					printf("%s", output_string);
				#endif
			#endif // __WXMSW__
 		}
 		else
 		{
				#if(wxCHECK_VERSION(3,0,0))
					output_string = _("The file does not exist.\n").c_str();
					printf("%s", output_string);
				#endif
 				return false;
 		}
 	}

	return false;}
