/***************************************************************
 * Name:      BilTrayApp.cpp
 * Purpose:   Code for Application Class
 * Author:    Igor Calì (igor.cali0@gmail.com)
 * Created:   2017-01-08
 * Copyright: Igor Calì (igor.cali0@gmail.com)
 * License:   GNU
 **************************************************************/

#ifndef BILTRAY_CPP_INCLUDED
#define BILTRAY_CPP_INCLUDED

#ifdef _BIL_USEUNITY
	#include <sys/msg.h>
#endif // _BIL_USEUNITY

#ifndef WX_PRECOMP
	#include <wx/wx.h>
#else
	#include <wx/wxprec.h>
#endif

#ifndef __WXMSW__
  #include <signal.h>
#endif // __WXMSW__

#include <wx/fileconf.h> // For wxFileConfig
#include <wx/snglinst.h>	// for wxSingleInstanceChecker
#include <wx/filename.h>
#include <wx/stdpaths.h> // For wxStandardPathsBase

#include "../platformsetup.h"
#include "BilTrayApp.h"
#include "../types.h"
//#include "../bil30opt.h"
//#include "../bil30core.h"
//#include "../../rsrc/icons/Icon_Application.xpm"

#ifdef __DEBIAN__
	#include "../../rsrc/icons/16/biltray_pale.xpm"
	#include "../../rsrc/icons/16/logo_disabled.xpm"
	#include "../../rsrc/icons/16/biltray_attention.xpm"
#else
	#include "../../rsrc/icons/biltray_pale.xpm"
	#include "../../rsrc/icons/Logo_disabled.xpm"
	#include "../../rsrc/icons/biltray_attention.xpm"
#endif // __DEBIAN__

#ifdef _BIL_USEUNITY
	#include "mailbox.h"
#endif // _BIL_USEUNITY

#ifdef __BORLANDC__
	#pragma hdrstop
#endif //__BORLANDC__

#ifdef _BIL_DEBUG
	const int MainTimerInt=10000;
#else
	const int MainTimerInt=600000;			// Data check timer
#endif // _BIL_DEBUG
	const int TermTimerInt=3000;				// Kill check timer
#ifdef _BIL_USEUNITY
	const int CommandTimerInt=1000;		// Command check timer (_BIL_USEUNITY)
#endif // _BIL_USEUNITY

// Code start
static const wxLanguage langIds[]={
	wxLANGUAGE_DEFAULT,
	wxLANGUAGE_ENGLISH,
	wxLANGUAGE_ITALIAN};

const wxString langNames[]={
	_("System language"),
	L"English",
  L"Italiano"};

// the arrays must be in sync
wxCOMPILE_TIME_ASSERT( WXSIZEOF(langNames)==WXSIZEOF(langIds),LangArraysMismatch);

wxLanguage Lan;

wxLocale bil_locale;

#ifndef __WXMSW__
  bool Killed=false;	// Set to true when SIGTERM is received
  wxTimer *CheckTimer;
#endif // __WXMSW__

#ifdef _BIL_USEUNITY
	bil_msgbuf command;
	wxTimer *CommandTimer;
#endif // _BIL_USEUNITY

bool HasAlarms=false;
wxFileConfig *INI;
//TOpt *GeneralOptions;
wxSingleInstanceChecker *m_checker_biltray;	// checks if another instance is already running
wxTimer *MainTimer;
TDataWithAlarms *Data;

IMPLEMENT_APP(BilTrayApp);

#ifndef __WXMSW__
  void term(int signum){	// Captures the SIGTERM signal
      Killed = true;}
#endif // __WXMSW__

bool BilTrayApp::OnInit()
{
	#ifdef __WXMSW__
    // TODO (igor#1#): implement Windows version
	#else
		wxString homecfg = GetUserConfigDir();
    homecfg+=L"/bilancio.ini";
    if(!::wxFileExists(homecfg)){
      wxFile *f=new wxFile(homecfg,wxFile::write);
      f->Create(homecfg,false,wxS_DEFAULT);
      f->Close();}
    INI=new wxFileConfig(wxEmptyString,wxEmptyString,homecfg,wxEmptyString,wxCONFIG_USE_LOCAL_FILE,wxConvAuto());
  #endif // __WXMSW__

	//GeneralOptions=new TOpt();

	#ifdef _BIL_DEBUG
		#ifdef __WXGTK__
				wxLocale::AddCatalogLookupPathPrefix(L".");
			#endif
	#endif

	#ifdef __WXMSW__
    wxLocale::AddCatalogLookupPathPrefix(GetShareDir());
	#else
		wxLocale::AddCatalogLookupPathPrefix(GetShareDir()+L"locale"/*/it/LC_MESSAGES"*/);
    		// NOTE (igor#1#): 2014/08/16 - Monitor the path at line above; it worked well previously with full path (now commented).
    		//                 Is it related to wxWidgets 3.0.1 installed in Ubuntu from tarball since debian packages not available?
	#endif // __WXMSW__
	bil_locale.Init(wxLANGUAGE_DEFAULT,wxLOCALE_LOAD_DEFAULT);
	bil_locale.AddCatalog(L"bilancio");
	Lan=FindLang();

	// Check if already running
	wxString user = wxGetUserId();
	const wxString name2 = wxString::Format(L"bilancio-%s",user.c_str());
	wxSingleInstanceChecker *m_checker_bilancio = new wxSingleInstanceChecker(name2);
	if ( m_checker_bilancio->IsAnotherRunning() ){
		wxMessageBox(_("Bilancio is already running!"),_("Bilancio"), wxICON_EXCLAMATION);
		return false;}
	else delete m_checker_bilancio;

	const wxString name = wxString::Format(L"biltray-%s",user.c_str());
	m_checker_biltray = new wxSingleInstanceChecker(name);
	if ( m_checker_biltray->IsAnotherRunning() ){
		wxMessageBox(_("Bilancio is already running!"),_("Bilancio"), wxICON_EXCLAMATION);
		return false;}

	Connect( wxEVT_TIMER, wxTimerEventHandler( BilTrayApp::TimerFire ) );

	wxString Document = GetDefaultDocument();
	if(Document.IsEmpty()){
		delete m_checker_biltray;
		return 0;}
	if(! wxFileExists(Document)){
		delete m_checker_biltray;
		return 0;}
	Data = new TDataWithAlarms();

	if(! Data->OpenDatabase(Document)){
		delete m_checker_biltray;
		return 0;}

	#ifdef _BIL_USEUNITY
		isUnity = false;
		wxString desktop;
		wxGetEnv(L"XDG_CURRENT_DESKTOP",&desktop);
		desktop.LowerCase();
		if(desktop == L"unity") isUnity = true;

		if(isUnity){
			indicator_killed = false;
			wxString Proc=wxGetCwd();
			#ifdef _BIL_DEBUG
				Proc += L"/Debug_GTK";
			#endif // _BIL_DEBUG
			Proc += L"/bilindicator";
			if(! wxExecute(Proc, wxEXEC_ASYNC))return false;

			//Create the Indicator Mailbox
			msqidInd = msgget(MAILBOX_INDICATOR, 0600 | IPC_CREAT);
			//Create the Biltray Mailbox
			msqidBil = msgget(MAILBOX_BILTRAY, 0600 | IPC_CREAT);}
		else
	#endif // _BIL_USEUNITY
	Tray=new MyTaskBarIcon();

	HasAlarms=Data->HasAlarms();
	#ifdef _BIL_USEUNITY
		if(isUnity){
			Act = HasAlarms;
			SendStatus();
			if(!HasAlarms){
				wxString title,message;
				title=_("Bilancio");
				message=_("There is no alarm set in your budget document");
				wxString proc=L"notify-send -i "+GetShareDir()+L"icons/bilancio.png "+title+L" \""+message+L"\"";
				wxExecute(proc,wxEXEC_ASYNC);}}
		else{
	#endif // _BIL_USEUNITY
		Tray->Act = HasAlarms;
		if(HasAlarms){
			#ifdef __WXMSW__
				Tray->SetIcon(wxIcon(biltray_pale_xpm),_("Alarm Management"));
			#else
				Tray->SetIcon(wxICON(biltray_pale),_("Alarm Management"));
			#endif // __WXMSW__
		}
		else{
			wxString title,message;
			title=_("Bilancio");
			message=_("There is no alarm set in your budget document");
			#ifdef __WXMSW__
				Tray->SetIcon(wxIcon(Logo_disabled_xpm),_("Alarm Management"));
				Tray->ShowBalloon(title,message,10000,NIIF_INFO);
			#else
				Tray->SetIcon(wxICON(Logo_disabled),_("Alarm Management"));
				wxString proc=L"notify-send -i "+GetShareDir()+L"icons/bilancio.png "+title+L" \""+message+L"\"";
				wxExecute(proc,wxEXEC_ASYNC);
			#endif
		}
	#ifdef _BIL_USEUNITY
		}
	#endif // _BIL_USEUNITY

	MainTimer=new wxTimer();
	MainTimer->SetOwner(this);
	if(HasAlarms)MainTimer->Start(MainTimerInt,false);

	#ifndef __WXMSW__
    // Prepare timer to check SIGTERM
    CheckTimer=new wxTimer();
    CheckTimer->SetOwner(this);
    CheckTimer->Start(TermTimerInt,false);
  #endif // __WXMSW__

  #ifdef _BIL_USEUNITY
  	if(isUnity){
			CommandTimer=new wxTimer();
			CommandTimer->SetOwner(this);
			CommandTimer->Start(CommandTimerInt,false);}
  #endif // _BIL_USEUNITY

	wxTheApp->MainLoop();
	return true;}

void BilTrayApp::TimerFire( wxTimerEvent& event ){
	switch(event.GetInterval()){
		case MainTimerInt:
			DoChecks();
			break;
		#ifndef __WXMSW__
      case TermTimerInt:
        CheckTimerFire();
        break;
    #endif // __WXMSW__

    #ifdef _BIL_USEUNITY
    	case CommandTimerInt:
    		if(! isUnity) break;
    		if (msgrcv( msqidInd, &command, sizeof(command), btCmd_do_action, IPC_NOWAIT) != -1){
    			wxString Proc;
    			switch(command.value){
						case cmd_toggle_enable:
							Act = !Act;
							SendStatus();
							break;
						case cmd_run_bilancio:
							if(Data->FileData.Modified) Data->database->ReleaseSavepoint(L"rollback");
							Proc = wxGetCwd();
							#ifdef _BIL_DEBUG
								Proc += L"/Debug_GTK";
							#endif // _BIL_DEBUG

							Proc = Proc + L"/bilancio";

							delete Data;
							delete m_checker_biltray;

							if(wxExecute(Proc,wxEXEC_ASYNC))KillIndicator(cmd_close);
							break;
						case cmd_open_options:
							if(ShowOptionsDialog()){
								wxString Document = GetDefaultDocument();
								if(Document == wxEmptyString){
									delete m_checker_biltray;
									KillIndicator(cmd_resume);
									term(SIGTERM);}
								else
									Data->OpenDatabase((Document));
							}
							bil_msgbuf resume_message;
							resume_message.mtype = btCmd_kill;
							resume_message.value = cmd_resume;
							msgsnd( msqidBil, &resume_message, sizeof(resume_message), 0);
							break;
						case cmd_close:
							KillIndicator(cmd_close);
						}
					}
    #endif // _BIL_USEUNITY
}}

void BilTrayApp::DoChecks(void)
{
	MainTimer->Stop();
	//Tray->Animated=true;
	#ifdef _BIL_USEUNITY
		bil_msgbuf attention_message;
		if(isUnity){
			attention_message.mtype = btCmd_send_status;
			attention_message.value = tray_attention;
			msgsnd( msqidBil, &attention_message, sizeof(attention_message), 0);}
		else{
	#endif // _BIL_USEUNITY
		Tray->ringing = true;
		#ifdef __WXMSW__
			Tray->SetIcon(wxIcon(biltray_attention_xpm),_("Alarm Management"));
		#else
			Tray->SetIcon(wxICON(biltray_attention),_("Alarm Management"));
		#endif // __WXMSW__
	#ifdef _BIL_USEUNITY
		}
	#endif // _BIL_USEUNITY

	Data->CheckLentObjects();
	Data->CheckBorrowedObjects();
	Data->CheckShopList();

	if(Data->FileData.Modified){
		Data->database->ReleaseSavepoint(L"rollback");
		Data->FileData.Modified = false;
		Data->database->Savepoint(L"rollback");}

	#ifdef _BIL_USEUNITY
		if(isUnity){
			attention_message.value = tray_enabled;
			msgsnd( msqidBil, &attention_message, sizeof(attention_message), 0);}
		else{
	#endif // _BIL_USEUNITY
		Tray->ringing = false;
		#ifdef __WXMSW__
			Tray->SetIcon(wxIcon(biltray_pale_xpm),_("Alarm Management"));
		#else
			Tray->SetIcon(wxICON(biltray_pale),_("Alarm Management"));
		#endif // __WXMSW__
	#ifdef _BIL_USEUNITY
		}
	#endif // _BIL_USEUNITY
	wxTheApp->Yield(false);
	MainTimer->Start(MainTimerInt,false);}

int BilTrayApp::OnExit(){
	delete Data;

	delete m_checker_biltray;
	return 0;}

#ifndef __WXMSW__
  void BilTrayApp::CheckTimerFire(){
    #ifdef __WXGTK__
    // Following code prepare the action to caputure SIGTERM signals
    struct sigaction action;
    memset(&action, 0, sizeof(struct sigaction));
    action.sa_handler = term;
    sigaction(SIGTERM, &action, NULL);
    //sigaction(SIGCHLD, &action, NULL);
    /*
    #elif __WXMSW__
      typedef void (*SignalHandlerPointer)(int);
      SignalHandlerPointer previousHandler;
      previousHandler = signal(SIGABRT, term);
    */
    #endif

    //int loop = 0;
    if(!Killed){
        //sleep(3);
        //printf("Finished loop run %d.\n", loop++);
          return;}

    // Exit code
    //CheckTimer->Stop();
    #ifdef _BIL_USEUNITY
    	if(isUnity){
				KillIndicator(cmd_close);
				wxTheApp->ExitMainLoop();
				wxTheApp->Exit();}
			else{
		#endif // _BIL_USEUNITY
			wxCommandEvent evt(wxEVT_NULL,0);
			Tray->ExitClick(evt);
		#ifdef _BIL_USEUNITY
			}
		#endif // _BIL_USEUNITY

    return;}
#endif // __WXMSW__

#ifdef _BIL_USEUNITY
	void BilTrayApp::SendStatus(void){
		bil_msgbuf init_message;
		init_message.mtype = btCmd_send_status;
		if(Act) init_message.value = tray_enabled;
		else init_message.value = tray_disabled;
		msgsnd( msqidBil, &init_message, sizeof(init_message), 0);}

	void BilTrayApp::KillIndicator(long cmd){
		//CommandTimer->Stop();

		if(indicator_killed) return;
		bil_msgbuf kill_message;
		kill_message.mtype = btCmd_kill;
		kill_message.value = cmd;
		msgsnd( msqidBil, &kill_message, sizeof(kill_message), 0);
		if(cmd == cmd_close){
			term(SIGTERM);
			indicator_killed = true;}}
#endif // _BIL_USEUNITY

BEGIN_EVENT_TABLE( MyTaskBarIcon, wxTaskBarIcon )
	EVT_MENU( bilAct, MyTaskBarIcon::ToggleAct )
	EVT_MENU( bilRun, MyTaskBarIcon::RunBilClick )
	EVT_MENU( bilOpt, MyTaskBarIcon::OptionsClick )
	//EVT_MENU( bilAut, MyTaskBarIcon::AuthorClick )
	//EVT_MENU( bilDon, MyTaskBarIcon::DonateClick )
	EVT_MENU( bilExi, MyTaskBarIcon::ExitClick )
END_EVENT_TABLE()

MyTaskBarIcon::MyTaskBarIcon(){
	/*
	Animated=false;
	#ifdef __WXMSW__
    icons.push_back( wxIcon(icon_1_xpm));
    icons.push_back( wxIcon(icon_2_xpm));
    icons.push_back( wxIcon(icon_3_xpm));
    icons.push_back( wxIcon(icon_4_xpm));
	#else
    icons.push_back( wxICON(icon_1));
    icons.push_back( wxICON(icon_2));
    icons.push_back( wxICON(icon_3));
    icons.push_back( wxICON(icon_4));
  #endif // __WXMSW__
	currentIcon = icons.begin();
	*/
	ringing = false;}

wxMenu *MyTaskBarIcon::CreatePopupMenu()
{
	wxMenu *Popup = new wxMenu();
	//if(!Animated){
	if(!ringing){
		wxMenuItem* Activated;
		Activated = new wxMenuItem( Popup, bilAct, wxString( _("Enabled") ) , wxEmptyString, wxITEM_CHECK );
		Popup->Append( Activated );
		Activated->Check(Act);
		Activated->Enable(HasAlarms);

		wxMenuItem* RunBil;
		RunBil = new wxMenuItem( Popup, bilRun, wxString( _("Start Bilancio") ) , wxEmptyString, wxITEM_NORMAL );
		Popup->Append( RunBil );

		Popup->AppendSeparator();

		wxMenuItem* Options;
		Options = new wxMenuItem( Popup, bilOpt, wxString( _("Options") ) , wxEmptyString, wxITEM_NORMAL );
		Popup->Append( Options );

		/*
		wxMenuItem* Author;
		Author = new wxMenuItem( Popup, bilAut, wxString( _("Author") ) , wxEmptyString, wxITEM_NORMAL );
		Popup->Append( Author );

		wxMenuItem* Donate;
		Donate = new wxMenuItem( Popup, bilDon, wxString( _("Donate") ) , wxEmptyString, wxITEM_NORMAL );
		Popup->Append( Donate );
    */

		wxMenuItem* Exit;
		Exit = new wxMenuItem( Popup, bilExi, wxString( _("Exit") ) , wxEmptyString, wxITEM_NORMAL );
		Popup->Append( Exit );}
	return Popup;
}

void MyTaskBarIcon::ToggleAct(wxCommandEvent& event){
  Act=!Act;
  if(Act){
  	MainTimer->Start(MainTimerInt,false);
  	#ifdef __WXMSW__
      SetIcon(wxIcon(biltray_pale_xpm),_("Alarm Management"));
    #else
      SetIcon(wxICON(biltray_pale),_("Alarm Management"));
    #endif // __WXMSW__
  }
  else{
  	MainTimer->Stop();
  	#ifdef __WXMSW__
      SetIcon(wxIcon(Logo_disabled_xpm),_("Alarm Management"));
    #else
      SetIcon(wxICON(Logo_disabled),_("Alarm Management"));
    #endif // __WXMSW__
}}

void MyTaskBarIcon::RunBilClick(wxCommandEvent& event){
	wxString Proc = wxEmptyString;
	wxStandardPaths std
		#if(wxCHECK_VERSION(3,0,0))
			= wxStandardPaths::Get()
		#endif
	;
  wxString exePath = std.GetExecutablePath();
  wxString path;
  wxFileName::SplitPath (exePath, &path, NULL, NULL, wxPATH_NATIVE);
  /*
  #ifdef __WXMSW__
    Proc = "\"";
  #endif // __WXMSW__
  */

  Proc = Proc + path;

  #ifdef __WXMSW__
    Proc = Proc + L"\\";
  #else
    Proc = Proc + L"/";
  #endif // __WXMSW__

  Proc = Proc + L"bilancio";

  #ifdef __WXMSW__
    Proc = Proc + L".exe";
  #endif // __WXMSW__

  /*
  #ifdef __WXMSW__
    Proc = Proc + L"\"";
  #endif // __WXMSW__
  */

	delete m_checker_biltray;
	if(wxExecute(Proc,wxEXEC_ASYNC)){
		RemoveIcon();}}

void MyTaskBarIcon::OptionsClick(wxCommandEvent& event){
  if(ShowOptionsDialog()){
    wxString Document = GetDefaultDocument();
    if(Document == wxEmptyString){
    	RemoveIcon();
    	delete m_checker_biltray;}
    else
   		Data->OpenDatabase(Document);}}

void MyTaskBarIcon::ExitClick(wxCommandEvent& event){
  #ifndef __WXMSW__
    delete m_checker_biltray;
  #endif // __WXMSW__
  RemoveIcon();

  #ifdef __WXMSW__
    wxWindow *frame = wxTheApp->GetTopWindow();
    frame->Close();
  #endif // __WXMSW__

  wxTheApp->ExitMainLoop();
  wxTheApp->Exit();}

/*
void MyTaskBarIcon::Anim(void){
	SetIcon( *currentIcon );
	++currentIcon;
	if( currentIcon == icons.end() )currentIcon = icons.begin();}
*/

#endif // BILTRAY_CPP_INCLUDED
