/***************************************************************
 * Name:      bilupdate.cpp
 * Purpose:   Code for Bilancio update
 * Author:    Igor Calì (igor.cali0@gmail.com)
 * Created:   2017-01-11
 * Copyright: Igor Calì (igor.cali0@gmail.com)
 * License:		GNU
 **************************************************************/

#ifndef BILUPDATE_CPP_INCLUDED
#define BILUPDATE_CPP_INCLUDED

#ifndef WX_PRECOMP
	#include <wx/wx.h>
#else
	#include <wx/wxprec.h>
#endif

#include <wx/fileconf.h> // For wxFileConfig
#ifdef _BIL_USEREGISTRY
  #include <wx/msw/registry.h>
#endif // _BIL_USEREGISTRY

#include "bilupdate.h"
#include "../types.h"
#include "../productversion.h"

#ifdef __BORLANDC__
	#pragma hdrstop
	#pragma argsused
#endif

// Code start

#ifdef __WXMSW__
  typedef HRESULT (WINAPI *UDTF)(LPVOID, LPCTSTR, LPCTSTR, DWORD, LPVOID);

  wxFileConfig *INI;
#else
  extern wxFileConfig *INI;
#endif // __WXMSW__

typedef enum TTupdType{
  utOriginal = 1,

  utBugFix = 10,
  utSecurity,
  utImprovement,
  utRebuild,

  utTranslationNew = 20,
  utTranslationUpdate,

  utSiteChanged = 30,
  utHelpImprovement,

  utInstallerBugFix = 40,
  utInstallerSecurity,
  utInstallImprovement,
  utInstallerRebuild,

  utWxUpdate =50
}TupdType;

void WXEXPORT fnCheckUpdate(void){
	wxString regstr;
	wxDateTime lcheck;
	#ifdef _BIL_USEREGISTRY
	wxString SWKey = GetSWKey();
	wxString Key=_T("HKEY_LOCAL_MACHINE\\")+SWKey;
	wxRegKey *K=new wxRegKey(wxRegKey::HKLM,SWKey,wxRegKey::WOW64ViewMode_Default);
	if(K->HasValue("LastUpdateCheck")){
		K->QueryValue("LastUpdateCheck",regstr);
		lcheck = Bil_StrToDate(regstr);
    if( !lcheck.IsEarlierThan( wxDateTime::Today() )){
			delete K;
			//ReturnValue=1;
			return;}}
	#else
	INI->SetPath(L"/Advanced");
	if(INI->HasEntry(L"LastUpdateCheck")){
		regstr=INI->Read(L"LastUpdateCheck",wxEmptyString);
		lcheck=Bil_StrToDate(regstr);
		if(lcheck>=::wxDateTime::Today().ResetTime())return;}
  #endif // _BIL_USEREGISTRY

	// Version update check
	#ifdef __WXMSW__
    //INI=new wxFileConfig(wxEmptyString,wxEmptyString,/*GetLanguageFile()*/wxEmptyString,wxEmptyString,wxCONFIG_USE_LOCAL_FILE);
    wxString AppPath=GetInstallationPath()+"\\serverinfo.ini";
	#else
    wxString AppPath = GetUserConfigDir() + L"/serverinfo.ini";
  #endif  // __WXMSW__

  // (code_download)
  // NOTE (igor#1#): Using wget since wxUrl and wxHTTP does not download properly
  #ifdef __WXMSW__
    wxString Proc = "\"" + GetInstallationPath() + L"\\wget\\wget.exe\" --no-check-certificate -O \"" + AppPath + L"\" " + GetVersionFile();
    wxExecute(Proc, wxEXEC_SYNC);
  #else
    wxString Proc = L"wget -O " + AppPath + L" " + GetVersionFile();
    wxExecute(Proc,wxEXEC_SYNC);
  #endif // __WXMSW__

	TupdType UpdateType;
	int Maj,Min,Rel,Bui, checksum=0;
	wxFileConfig *UpINI=new wxFileConfig(wxEmptyString,wxEmptyString,AppPath,wxEmptyString,wxCONFIG_USE_LOCAL_FILE);
	TVersion instVer;
	long ver;
	UpINI->SetPath(L"VersionInfo");
	UpINI->Read(L"UpdateType",&ver,0);
	UpdateType=(TupdType)ver;
	UpINI->Read(L"Version",&ver,0);
	Maj = (int) ver;
	checksum += ver;
	UpINI->Read(L"SubVersion",&ver,0);
	Min = (int) ver;
	checksum += ver;
	UpINI->Read(L"Release",&ver,0);
	Rel = (int) ver;
	checksum += ver;
	UpINI->Read(L"Build",&ver,0);
	Bui = (int) ver;
	checksum += ver;

	unsigned int os_mask;
	unsigned int os;
	#ifdef __WXGTK__
		os_mask = 0x01;
	#elif (defined __WXMSW__)
		os = 0x02;
	#endif // __WXGTK__
	UpINI->Read(L"OS", &ver, 0);
	os = (unsigned int) ver;
	os = os & os_mask;

	delete UpINI;

	if (os == os_mask){
		if(checksum > 0){

			instVer=GetInstalledVersion();
			if((Maj>instVer.Maj) || (Min>instVer.Min) || (Rel>instVer.Rel) || (Bui>instVer.Bui)){
				wxString Msg,type,label;
				type=_("Update type: ");
				switch(UpdateType){
					case utOriginal:
						label=_("New version");
						type+=label;
						break;
					case utBugFix:
						label=_("Bugfix");
						type+=label;
						break;
					case utSecurity:
						label=_("Security fix");
						type+=label;
						break;
					case utImprovement:
						label=_("Improved features");
						type+=label;
						break;
					case utRebuild:
						label=_("Rebuild");
						type+=label;
						break;
					case utInstallerBugFix:
						type+=_("Bugfix (Installer)");
						break;
					case utInstallerSecurity:
						type+=_("Security fix (Installer)");
						break;
					case utInstallImprovement:
						type+=_("Improved features (Installer)");
						break;
					case utInstallerRebuild:
						type+=_("Rebuild (Installer)");
						break;
					case utSiteChanged:
						type+=_("Website update");
						break;
					case utHelpImprovement:
						type+=_("Improved help");
						break;
					case utWxUpdate:
						type+=_("wxWidgets library update");
						break;
					case utTranslationNew:
						type+=_("New translation");
						break;
					case utTranslationUpdate:
						type+=_("Translation update");
						break;
					default:
						type+=_("Unclassified");}
				Msg=_("A new version of Bilancio is available.\nDo you want to open the website to download it?")+L"\n\n"+type;

				//wxMessageDialog *Dialog=new wxMessageDialog(wxTheApp->GetTopWindow(),Msg,_("Bilancio"),wxYES_NO);
				if(wxMessageBox(Msg, _("Bilancio"), wxYES_NO) == wxYES) wxLaunchDefaultBrowser(GetWebAddress());
			}
		}
	}

	if(::wxFileExists(AppPath))::wxRemoveFile(AppPath);

	if(checksum){
    lcheck=::wxDateTime::Today();
    #ifdef __WXMSW__
      regstr=lcheck.Format("%d/%m/%Y",wxDateTime::Local);
    #else
      regstr=lcheck.Format(L"%d/%m/%Y",wxDateTime::Local);
    #endif // __WXMSW__
    #ifdef __WXMSW__
      #ifdef _BIL_USEREGISTRY
        K->SetValue("LastUpdateCheck",regstr);
      #endif // _BIL_USEREGISTRY
    #else
      INI->Write(L"LastUpdateCheck",regstr);
      INI->Flush();
    #endif
	}
}

#endif // BILUPDATE_CPP_INCLUDED
