/***************************************************************
 * Name:      igiberr.cpp
 * Purpose:   Code for Bilancio Application Class
 * Author:    Igor Calì (igor.cali0@gmail.com)
 * Created:   2016-11-15
 * Copyright: Igor Calì (igor.cali0@gmail.com)
 * License:		GNU
 **************************************************************/

#ifndef WX_PRECOMP
	#include <wx/wx.h>
#else
	#include <wx/wxprec.h>
#endif

#include <wx/file.h>
#include <wx/wxsqlite3.h>

#ifndef __BILCONVERT_BIN__
	#include <wx/dlimpexp.h>
#endif // __BILCONVERT_BIN__

#include "../platformsetup.h"
#include "../constants.h"

#include "../platformsetup.h"

#ifndef __BILCONVERT_BIN__
	#include "igiberr.h"
#endif // __BILCONVERT_BIN__

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#ifdef __WXGTK__
	extern wxString Console;
#endif // __WXGTK__

#if(! wxCHECK_VERSION(3, 0, 0))
	#include <wx/filename.h>    // For wxFileName
#endif	// wxCHECK_VERSION

void Error(int Err,wxString Opt){
	wxString Msg;
	switch(Err){
		case 1:
			Msg=::wxString::Format(_("The file %s is already opened!"),Opt.c_str());
			break;
		case 2:
			Msg=::wxString::Format(_("The file %s does not contain any valid budget data!"),Opt.c_str());
			break;
		case 3:
			Msg=_("The file is already in Bil2.x format!");
			break;
		case 4:
			Msg=_("Wrong password!");
			break;
		case 5:
			Msg=_("You must select a debt!");
			break;
		case 6:
			Msg=_("Configuration error! It is necessary to reinstall the application.");
			break;
		case 8:
			Msg=_("No fund can be removed!");
			break;
		case 9:
			Msg=_("No fund can be modified!");
			break;
		case 10:
			Msg=_("Error in budget data!");
			break;
		case 11:
			Msg=::wxString::Format(_("The fund %s already exists!"),Opt.c_str());
			break;
		case 12:
			Msg=_("The fund value must be less or equal to the default fund!");
			break;
		case 13:
			Msg=_("Not enough money for the operation!");
			break;
		case 14:
			Msg=_("No lent object!");
			break;
		case 15:
			Msg=_("No borrowed object!");
			break;
		case 16:
			Msg=_("No item in the shopping list!");
			break;
		case 17:
			Msg=_("This operation is only valid to increase the budget!");
			break;
		case 18:
			Msg=_("Error in budget data! Check that the default fund is set.");
			break;
		case 19:
			Msg=_("Not possible to set a credit greater than the default fund!");
			break;
		case 20:
			Msg=_("No credit stored!");
			break;
		case 21:
			Msg=::wxString::Format(_("Inserted value is greater than the chosen %s!"),Opt.c_str());
			break;
		case 22:
			Msg=_("No debt stored!");
			break;
		case 23:
			Msg=_("No fund selected!");
			break;
		case 24:
			Msg=_("Not enough money in the default fund!");
			break;
		case 25:
			Msg=_("You must specify a value!");
			break;
		case 26:
			Msg=_("You must insert a numeric value!");
			break;
		case 27:
			Msg=_("You must insert a value greater than zero!");
			break;
		/*
			case 28:
				Msg=_("Not possible to open last file!");
				break;
		*/
		case 29:
			Msg=_("You must insert a reason!");
			break;
		case 30:
			Msg=_("You must insert a name!");
			break;
		case 31:
			Msg=_("No item selected in the shopping list!");
			break;
		case 32:
			Msg=_("You must insert an object!");
			break;
		case 33:
			Msg=_("You must insert a contact!");
			break;
		case 34:
			Msg=_("Two labels are identical!");
			break;
		/*
		case 35:
			Msg=_("Unexisting folder! Insert a valid path or click the Browse button");
			break;
		*/
		case 36:
			Msg=_("Cannot open the file!");
			break;
		/*
		case 37:
			Msg=_("Not possible to sort data!");
			break;
		*/
		case 38:
			Msg=_("Empty string!");
			break;
		case 39:
			Msg=_("You must select a credit!");
			break;
		case 40:
			Msg=_("You must insert the password!");
			break;
		case 41:
			Msg=::wxString::Format(_("File %s is read-only!"),Opt.c_str());
			break;
		case 42:
			Msg=_("You must use this application from Bilancio application!");
			break;
		case 43:
			Msg=_("You can convert only one file at a time!");
			break;
		case 44:
			Msg=_("Error during the document conversion!");
			break;
		case 45:
			Msg=::wxString::Format(_("Not possible to find file %s"),Opt.c_str());
			break;
		case 46:
			Msg = _("Invalid date");
			break;
		case 47:
			Msg = _("You have to select a file name!");
			break;
		case 48:
			Msg=::wxString::Format(_("The item %s already exists!"),Opt.c_str());
			break;
		default:
			Msg=_("Unknown error");};
  #ifndef __BILCONVERT_BIN__
    #ifdef __WXMSW__
      MessageBeep(MB_ICONHAND);
    #else
        wxBell();
    #endif // __WXMSW__
	#endif // __BILCONVERT_BIN__

  #ifndef __BILCONVERT_BIN__
  	wxMessageBox(Msg,_T("Bilancio"),wxICON_ERROR);
	#else
		#if(wxCHECK_VERSION(3,0,0))
			const char *output_string;
			output_string = Msg.c_str();
			printf("%s", output_string);
		#endif
	#endif // __BILCONVERT_BIN__
	}

#ifdef _BIL_FORMAT2x
	bool Check2x(wxString File){
		bool Ret;
		//int FH,BR;	// FH: file handler

									// BR: buffer lenght
		wxFileOffset FL;	// FL: file lenght
		char *Buffer;
		wxString S=wxEmptyString;
		wxFile *fh=new wxFile(File,wxFile::read);
		if(!fh->IsOpened()){
			delete fh;
			return false;}
		//FL=fh->Len();
		FL=fh->SeekEnd(0);
		fh->Seek(0);
		Buffer=new char[FL+1];
		if(fh->Read(Buffer,6)==6)
			//S=Buffer;
		{
			for(int i=0;i<6;i++)S+=Buffer[i];
		}
		else{
			fh->Close();
			delete fh;
			return false;}
		fh->Close();
		delete fh;
		delete[] Buffer;
		if(S==L"Bil2.0")Ret=true;
		else if(S==L"Bil2.2")Ret=true;
		else if(S==L"Bil2.3")Ret=true;
		else Ret=false;
		return Ret;}
#endif // _BIL_FORMAT2x

bool Check301(wxString File){
	bool Ret;
	wxFileOffset FL;	// FL: file lenght
	char *Buffer;
	wxString S = wxEmptyString;
	wxFile *fh = new wxFile(File, wxFile::read);
	if(!fh->IsOpened()){
		delete fh;
		return false;}
	FL=fh->SeekEnd(0);
	fh->Seek(0);
	Buffer=new char[FL + 1];
	if(fh->Read(Buffer, 6) == 6) for(int i=0; i<6; i++) S += Buffer[i];
	else{
		fh->Close();
		delete fh;
		return false;}
	fh->Close();
	delete fh;
	delete[] Buffer;
	if(S == L"Bil3.0") Ret = true;
	else Ret = false;
	return Ret;}

bool Check302(wxString File){
	bool Ret;
	wxFileOffset FL;	// FL: file lenght
	char *Buffer;
	wxString S = wxEmptyString;
	wxFile *fh = new wxFile(File, wxFile::read);
	if(!fh->IsOpened()){
		delete fh;
		return false;}
	FL=fh->SeekEnd(0);
	fh->Seek(0);
	Buffer=new char[FL + 1];
	if(fh->Read(Buffer, 6) == 6) for(int i=0; i<6; i++) S += Buffer[i];
	else{
		fh->Close();
		delete fh;
		return false;}
	fh->Close();
	delete fh;
	delete[] Buffer;
	if(S == L"Bil302") Ret = true;
	else Ret = false;
	return Ret;}

bool Check31(wxString file){
	wxSQLite3Database *db = new wxSQLite3Database();
	db->Open(file, wxEmptyString, WXSQLITE_OPEN_READWRITE | WXSQLITE_OPEN_CREATE );
	try{
		wxSQLite3ResultSet test = db->ExecuteQuery("pragma schema_version;");

		int version = test.GetInt(0);
		db->Close();

		if(version >= 0) return true;	// 0: pre-existing empty file
																	// 1 or greater: schema updates (existing database
																	// http://www.tutorialspoint.com/sqlite/sqlite_pragma.htm
		else return false;
	}
	catch(...){
		return false;
	}}

int CheckFileFormat(wxString File){
	int Ret;

	if(Check31(File)) Ret = bilFFORMAT_31;
	else

	if(Check302(File)) Ret = bilFFORMAT_302;
	else if(Check301(File))Ret = bilFFORMAT_301;
	#ifdef _BIL_FORMAT2x
		else if(Check2x(File))Ret = bilFFORMAT_2;
	#endif // _BIL_FORMAT2x
	else Ret = bilFFORMAT_UNKNOWN;
	return Ret;}

#ifndef __BILCONVERT_BIN__
	bool CheckAndPromptForConversion(wxString File){
		wxString Name;
		switch(CheckFileFormat(File)){
			case bilFFORMAT_UNKNOWN:	// Unknown Format
				Error(2, File);
				return false;
			case bilFFORMAT_31:
				break;
			#ifdef _BIL_FORMAT2x
				case 2:	// v2
			#endif // _BIL_FORMAT2x
			case bilFFORMAT_302:
			case bilFFORMAT_301:	// v3
				::wxFileName::SplitPath(File, NULL, NULL, &Name, NULL, wxPATH_NATIVE);
				#if(wxCHECK_VERSION(3,0,0))
					if(wxMessageBox(wxString::Format(_("The format of file %s is old.\nDo you want to convert it?"), Name), _("Bilancio"), wxYES_NO) == wxYES){
				#else
					if(wxMessageBox(wxString::Format(_("The format of file %s is old.\nDo you want to convert it?"), Name.c_str()), _("Bilancio"), wxYES_NO) == wxYES){
				#endif
					wxString Proc;
					#ifdef __WXMSW__
						Proc = "\"" + GetShareDir() + L"\\bilconvert.exe\" \"" + File + "\"";
						wxExecute(Proc, wxEXEC_SYNC | wxEXEC_SHOW_CONSOLE);
					#else
						Proc = Console + wxGetCwd() + L"/bilconvert \"" + File + L"\"";
						wxExecute(Proc, wxEXEC_SYNC);
					#endif
					if(CheckFileFormat(File) != bilFFORMAT_31) return false;}
				else return false;
				break;
			default:
				return false;}
		return true;
	}
#endif // __BILCONVERT_BIN__

